#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Ansible module to manage CheckPoint Firewall (c) 2019
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#

from __future__ import (absolute_import, division, print_function)

__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = """
---
module: cp_mgmt_mobile_profile
short_description: Manages mobile-profile objects on Checkpoint over Web Services API
description:
  - Manages mobile-profile objects on Checkpoint devices including creating, updating and removing objects.
  - All operations are performed over Web Services API.
version_added: "6.0.0"
author: "Eden Brillant (@chkp-edenbr)"
options:
  name:
    description:
      - Object name.
    type: str
    required: True
  applications:
    description:
      - Applications settings.
    type: dict
    suboptions:
      enable_print_mails:
        description:
          - Allow to print mails.
        type: bool
      max_attachments_size:
        description:
          - Maximum size of attachments allowed for downloading -  you can choose a unit (gbs, kbs, mbs, bytes) in "max-attachments-unit" field.
        type: int
      calendar_from_the_last:
        description:
          - How far back to see your Calendar from the current date - you can choose a unit (day, week, month) in "calendar-from-the-last-unit" field.
        type: int
      calendar_from_the_last_unit:
        description:
          - Unit for "calendar-from-the-last" numeric value.
        type: str
        choices: ['weeks', 'months', 'days']
      calendar_to_the_following:
        description:
          - How much ahead to see your Calendar from the current date - you can choose a unit (day, week, month) in "calendar-to-the-following-unit" field.
        type: int
      calendar_to_the_following_unit:
        description:
          - Unit for "calendar-to-the-following" numeric value.
        type: str
        choices: ['weeks', 'months', 'days']
      mail_from_the_last:
        description:
          - How far back to see your emails from the current date - choose a unit (day, week, month) in "mail-from-the-last-unit" field.
        type: int
      mail_from_the_last_unit:
        description:
          - Unit for "mail-from-the-last" numeric value.
        type: str
        choices: ['weeks', 'months', 'days']
      synchronize_contacts:
        description:
          - Contacts synchronization method - from the mail server to device and the app and vice versa or from the mail server to device and the
            app or from the mail server to the app.
        type: str
        choices: ['mail srv to app and device and vice versa', 'mail srv to app and device', 'mail srv to app']
      allow_push_notification:
        description:
          - Allow to receive push notifications of mails and meetings.
        type: bool
      allow_calendar_sync:
        description:
          - Allow synchronization between business calendar to device calendar.
        type: bool
      allow_contacts_from_global_address_list:
        description:
          - Allow to add additional contacts from Global Address List to the app.
        type: bool
      allow_contacts_from_local_phone:
        description:
          - Allow to add additional contacts from local phone to the app.
        type: bool
      save_local_web_cache:
        description:
          - Configure whether local cache data generated by web browser should be preserved.
        type: bool
      allow_caching_docsec_credentials:
        description:
          - Allow store encrypted document credentials in application secure storage.
        type: bool
      allow_caching_docsec_keys:
        description:
          - Allow store encrypted document keys in application secure storage.
        type: bool
  client_customization:
    description:
      - Client customization settings.
    type: dict
    suboptions:
      app_theme_color_dark:
        description:
          - Configure the application display colors in Dark mode. 6 hex digits that define RGB color - relevant for IOS.
        type: str
      app_theme_color_light:
        description:
          - Configure the application display colors in light mode. 6 hex digits that define RGB color - relevant for IOS.
        type: str
      allow_calendar:
        description:
          - Allow sync business calendar to device calendar.
        type: bool
      allow_contacts:
        description:
          - Enable/Disable contacts app.
        type: bool
      allow_mail:
        description:
          - Enable/Disable email app.
        type: bool
      allow_notes_sync:
        description:
          - Allow sync business notes to device notes.
        type: bool
      allow_saved_file_apps:
        description:
          - Allow the appearance of 'Saved file app' in the app list.
        type: bool
      allow_secure_chat:
        description:
          - Enable/Disable Messages app (depends on Mail app).
        type: bool
      allow_tasks:
        description:
          - Enable/Disable Tasks app.
        type: bool
      certificate_expire_message:
        description:
          - message to show users when certificate is expired - for admin to fill - can contain only English characters, digits, comma, spaces and points.
        type: str
  data_leak_prevention:
    description:
      - Data leak prevention settings.
    type: dict
    suboptions:
      open_extension_with_external_app:
        description:
          - Open the following extensions from your app with external apps when they cannot be opened with Capsule viewer.
        type: list
        elements: str
        choices: ['any file', 'word documents', 'excel documents', 'powerpoint documents', 'any microsoft office documents', 'media files']
      share_protected_extension:
        description:
          - Share protected files extensions to external apps.
        type: list
        elements: str
        choices: ['any file', 'word documents', 'excel documents', 'powerpoint documents', 'any microsoft office documents', 'media files']
      share_unprotected_extension:
        description:
          - Share unprotected files extensions to external apps.
        type: list
        elements: str
        choices: ['any file', 'word documents', 'excel documents', 'powerpoint documents', 'any microsoft office documents', 'media files']
      allow_copy_paste:
        description:
          - Allow copy paste of mail content.
        type: bool
      block_forward_attachments:
        description:
          - Allow share mail attachments with external mails.
        type: bool
      block_screenshot:
        description:
          - If true - you can't make a screenshot from your app.
        type: bool
      allowed_domains_forward_attachment:
        description:
          - exclusion of domains which attachments are allowed to be sent, even that shared policy prevents sharing these kinds of attached files
            - can contain only English characters, digits, comma, spaces and points.
        type: str
      accept_protected_file_extensions:
        description:
          - Accept protected files with these extensions from external apps to your app.
        type: list
        elements: str
        choices: ['any file', 'word documents', 'excel documents', 'powerpoint documents', 'any microsoft office documents', 'media files']
      accept_unprotected_file_extensions:
        description:
          - Accept unprotected files with these extensions from external apps to your app.
        type: list
        elements: str
        choices: ['any file', 'word documents', 'excel documents', 'powerpoint documents', 'any microsoft office documents', 'media files']
      allow_import_from_gallery:
        description:
          - Allow import media from gallery.
        type: bool
      allow_taking_photos_and_videos:
        description:
          - Allow the camera to be used from your app.
        type: bool
      offer_capsule_as_viewer:
        description:
          - Offer Capsule as a viewer for external protected documents.
        type: bool
  harmony_mobile:
    description:
      - Integrations settings.
    type: dict
    suboptions:
      protect_policy_enabled:
        description:
          - Enable/disable Protect Application- cannot be enable if Harmony SDK is enable.
        type: bool
      protect_high_risk_action:
        description:
          - What is the action if there is high risk found by Harmony Mobile.
        type: str
        choices: ['none', 'wipe', 'block']
      protect_high_risk_message:
        description:
          - The message can contain only English characters, digits, comma, spaces and points.
        type: str
      protect_medium_risk_action:
        description:
          - What is the action if there is medium risk found by Harmony Mobile.
        type: str
        choices: ['none', 'wipe', 'block']
      protect_medium_risk_message:
        description:
          - The message can contain only English characters, digits, comma, spaces and points.
        type: str
      protect_not_activated_action:
        description:
          - What is the action if there is policy violation (configuration for Harmony Mobile).
        type: str
        choices: ['none', 'wipe', 'block']
      protect_not_activated_message:
        description:
          - The message can contain only English characters, digits, comma, spaces and points.
        type: str
      enable_harmony_mobile_sdk:
        description:
          - Enable/disable Harmony SDK - cannot be enable if Harmony Mobile Application is enable.
        type: bool
      compromised_behavior:
        description:
          - Device configuration - response to malicious behavior (configuration for Harmony SDK).
        type: str
        choices: ['block', 'notify', 'ignore']
      harmony_mobile_sdk_license:
        description:
          - License for Harmony Mobile Sdk (configuration for Harmony SDK) - can contain only English characters, digits, comma, spaces and point.
        type: str
      malware_behavior:
        description:
          - Behavior when App is identified as malicious (configuration for Harmony SDK).
        type: str
        choices: ['block', 'notify', 'ignore']
      man_in_the_middle_attack:
        description:
          - Behavior when there is a network man-in-the-middle attack (configuration for Harmony SDK).
        type: str
        choices: ['block', 'notify', 'ignore']
      os_integrity_compromised:
        description:
          - Behavior when Device OS is compromised (configuration for Harmony SDK).
        type: str
        choices: ['block', 'notify', 'ignore']
      suspicious_app:
        description:
          - Behavior when App is suspected as malicious (configuration for Harmony SDK).
        type: str
        choices: ['block', 'notify', 'ignore']
      suspicious_enterprise_certificate:
        description:
          - Behavior when a certificate profile has been installed allowing the installing of apps on device from unknown source - iOS only
            (configuration for Harmony SDK).
        type: str
        choices: ['block', 'notify', 'ignore']
  security:
    description:
      - Security settings.
    type: dict
    suboptions:
      session_timeout:
        description:
          - Session timeout - you can choose a unit (day, week, month) in "session-timeout-unit" field.
        type: int
      session_timeout_unit:
        description:
          - Unit for "session-timeout" numeric value.
        type: str
        choices: ['weeks', 'days', 'hours', 'minutes']
      activate_passcode_lock:
        description:
          - Require passcode to the application.
        type: bool
      allow_store_credentials:
        description:
          - Allow storing the credentials on the device.
        type: bool
      passcode_profile:
        description:
          - Passcode Policy object identified by the name or UID.
        type: str
      report_jailbroken:
        description:
          - Issue log when device is detected as jail broken.
        type: bool
      block_jailbroken:
        description:
          - Action upon detection of jail broken devices.
        type: str
        choices: ['block', 'none']
      block_3rd_party_keyboard:
        description:
          - Block 3rd party keyboard.
        type: bool
      hide_ssl_connect_anyway_button:
        description:
          - Hide connect button on critical SSL trust failures.
        type: bool
  tags:
    description:
      - Collection of tag identifiers.
    type: list
    elements: str
  color:
    description:
      - Color of the object. Should be one of existing colors.
    type: str
    choices: ['aquamarine', 'black', 'blue', 'crete blue', 'burlywood', 'cyan', 'dark green', 'khaki', 'orchid', 'dark orange', 'dark sea green',
             'pink', 'turquoise', 'dark blue', 'firebrick', 'brown', 'forest green', 'gold', 'dark gold', 'gray', 'dark gray', 'light green', 'lemon chiffon',
             'coral', 'sea green', 'sky blue', 'magenta', 'purple', 'slate blue', 'violet red', 'navy blue', 'olive', 'orange', 'red', 'sienna', 'yellow']
  comments:
    description:
      - Comments string.
    type: str
  details_level:
    description:
      - The level of detail for some of the fields in the response can vary from showing only the UID value of the object to a fully detailed
        representation of the object.
    type: str
    choices: ['uid', 'standard', 'full']
  domains_to_process:
    description:
      - Indicates which domains to process the commands on. It cannot be used with the details-level full, must be run from the System Domain only and
        with ignore-warnings true. Valid values are, CURRENT_DOMAIN, ALL_DOMAINS_ON_THIS_SERVER.
    type: list
    elements: str
  ignore_warnings:
    description:
      - Apply changes ignoring warnings.
    type: bool
  ignore_errors:
    description:
      - Apply changes ignoring errors. You won't be able to publish such a changes. If ignore-warnings flag was omitted - warnings will also be ignored.
    type: bool
extends_documentation_fragment: check_point.mgmt.checkpoint_objects
"""

EXAMPLES = """
- name: add-mobile-profile
  cp_mgmt_mobile_profile:
    name: New Mobile Profile
    state: present

- name: set-mobile-profile
  cp_mgmt_mobile_profile:
    data_leak_prevention:
      share_protected_extension: word documents
    name: New Mobile Profile
    state: present

- name: delete-mobile-profile
  cp_mgmt_mobile_profile:
    name: New Mobile Profile
    state: absent
"""

RETURN = """
cp_mgmt_mobile_profile:
  description: The checkpoint object created or updated.
  returned: always, except when deleting the object.
  type: dict
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.check_point.mgmt.plugins.module_utils.checkpoint import checkpoint_argument_spec_for_objects, api_call


def main():
    argument_spec = dict(
        name=dict(type='str', required=True),
        applications=dict(type='dict', options=dict(
            enable_print_mails=dict(type='bool'),
            max_attachments_size=dict(type='int'),
            calendar_from_the_last=dict(type='int'),
            calendar_from_the_last_unit=dict(type='str', choices=['weeks', 'months', 'days']),
            calendar_to_the_following=dict(type='int'),
            calendar_to_the_following_unit=dict(type='str', choices=['weeks', 'months', 'days']),
            mail_from_the_last=dict(type='int'),
            mail_from_the_last_unit=dict(type='str', choices=['weeks', 'months', 'days']),
            synchronize_contacts=dict(type='str', choices=['mail srv to app and device and vice versa', 'mail srv to app and device', 'mail srv to app']),
            allow_push_notification=dict(type='bool'),
            allow_calendar_sync=dict(type='bool'),
            allow_contacts_from_global_address_list=dict(type='bool'),
            allow_contacts_from_local_phone=dict(type='bool'),
            save_local_web_cache=dict(type='bool'),
            allow_caching_docsec_credentials=dict(type='bool'),
            allow_caching_docsec_keys=dict(type='bool')
        )),
        client_customization=dict(type='dict', options=dict(
            app_theme_color_dark=dict(type='str'),
            app_theme_color_light=dict(type='str'),
            allow_calendar=dict(type='bool'),
            allow_contacts=dict(type='bool'),
            allow_mail=dict(type='bool'),
            allow_notes_sync=dict(type='bool'),
            allow_saved_file_apps=dict(type='bool'),
            allow_secure_chat=dict(type='bool'),
            allow_tasks=dict(type='bool'),
            certificate_expire_message=dict(type='str')
        )),
        data_leak_prevention=dict(type='dict', options=dict(
            open_extension_with_external_app=dict(type='list', elements='str', choices=['any file', 'word documents',
                                                                                        'excel documents',
                                                                                        'powerpoint documents',
                                                                                        'any microsoft office documents',
                                                                                        'media files']),
            share_protected_extension=dict(type='list', elements='str', choices=['any file', 'word documents', 'excel documents',
                                                                                 'powerpoint documents',
                                                                                 'any microsoft office documents', 'media files']),
            share_unprotected_extension=dict(type='list', elements='str', choices=['any file', 'word documents', 'excel documents',
                                                                                   'powerpoint documents',
                                                                                   'any microsoft office documents', 'media files']),
            allow_copy_paste=dict(type='bool'),
            block_forward_attachments=dict(type='bool'),
            block_screenshot=dict(type='bool'),
            allowed_domains_forward_attachment=dict(type='str'),
            accept_protected_file_extensions=dict(type='list', elements='str', choices=['any file', 'word documents', 'excel documents',
                                                                                        'powerpoint documents',
                                                                                        'any microsoft office documents', 'media files']),
            accept_unprotected_file_extensions=dict(type='list', elements='str', choices=['any file', 'word documents', 'excel documents',
                                                                                          'powerpoint documents',
                                                                                          'any microsoft office documents', 'media files']),
            allow_import_from_gallery=dict(type='bool'),
            allow_taking_photos_and_videos=dict(type='bool'),
            offer_capsule_as_viewer=dict(type='bool')
        )),
        harmony_mobile=dict(type='dict', options=dict(
            protect_policy_enabled=dict(type='bool'),
            protect_high_risk_action=dict(type='str', choices=['none', 'wipe', 'block']),
            protect_high_risk_message=dict(type='str'),
            protect_medium_risk_action=dict(type='str', choices=['none', 'wipe', 'block']),
            protect_medium_risk_message=dict(type='str'),
            protect_not_activated_action=dict(type='str', choices=['none', 'wipe', 'block']),
            protect_not_activated_message=dict(type='str'),
            enable_harmony_mobile_sdk=dict(type='bool'),
            compromised_behavior=dict(type='str', choices=['block', 'notify', 'ignore']),
            harmony_mobile_sdk_license=dict(type='str'),
            malware_behavior=dict(type='str', choices=['block', 'notify', 'ignore']),
            man_in_the_middle_attack=dict(type='str', choices=['block', 'notify', 'ignore']),
            os_integrity_compromised=dict(type='str', choices=['block', 'notify', 'ignore']),
            suspicious_app=dict(type='str', choices=['block', 'notify', 'ignore']),
            suspicious_enterprise_certificate=dict(type='str', choices=['block', 'notify', 'ignore'])
        )),
        security=dict(type='dict', options=dict(
            session_timeout=dict(type='int'),
            session_timeout_unit=dict(type='str', choices=['weeks', 'days', 'hours', 'minutes']),
            activate_passcode_lock=dict(type='bool'),
            allow_store_credentials=dict(type='bool'),
            passcode_profile=dict(type='str', no_log=False),
            report_jailbroken=dict(type='bool'),
            block_jailbroken=dict(type='str', choices=['block', 'none']),
            block_3rd_party_keyboard=dict(type='bool'),
            hide_ssl_connect_anyway_button=dict(type='bool')
        )),
        tags=dict(type='list', elements='str'),
        color=dict(type='str', choices=['aquamarine', 'black', 'blue', 'crete blue', 'burlywood', 'cyan', 'dark green',
                                        'khaki', 'orchid', 'dark orange', 'dark sea green', 'pink', 'turquoise', 'dark blue', 'firebrick', 'brown',
                                        'forest green', 'gold', 'dark gold', 'gray', 'dark gray', 'light green', 'lemon chiffon', 'coral', 'sea green',
                                        'sky blue', 'magenta', 'purple', 'slate blue', 'violet red', 'navy blue', 'olive', 'orange', 'red', 'sienna',
                                        'yellow']),
        comments=dict(type='str'),
        details_level=dict(type='str', choices=['uid', 'standard', 'full']),
        domains_to_process=dict(type='list', elements='str'),
        ignore_warnings=dict(type='bool'),
        ignore_errors=dict(type='bool')
    )
    argument_spec.update(checkpoint_argument_spec_for_objects)

    module = AnsibleModule(argument_spec=argument_spec, supports_check_mode=True)
    api_call_object = 'mobile-profile'

    result = api_call(module, api_call_object)
    module.exit_json(**result)


if __name__ == '__main__':
    main()



#
# -*- coding: utf-8 -*-
# Copyright 2019 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
"""
The iosxr_lldp_global class
It is in this file where the current configuration (as dict)
is compared to the provided configuration (as dict) and the command set
necessary to bring the current configuration to it's desired end-state is
created
"""

from __future__ import absolute_import, division, print_function


__metaclass__ = type


from ansible.module_utils.six import iteritems
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.cfg.base import (
    ConfigBase,
)
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.utils import (
    dict_diff,
    remove_empties,
    to_list,
)

from ansible_collections.cisco.iosxr.plugins.module_utils.network.iosxr.facts.facts import Facts
from ansible_collections.cisco.iosxr.plugins.module_utils.network.iosxr.utils.utils import (
    dict_delete,
    flatten_dict,
)


class Lldp_global(ConfigBase):
    """
    The iosxr_lldp class
    """

    gather_subset = ["!all", "!min"]

    gather_network_resources = ["lldp_global"]

    def __init__(self, module):
        super(Lldp_global, self).__init__(module)

    def get_lldp_global_facts(self, data=None):
        """Get the 'facts' (the current configuration)

        :rtype: A dictionary
        :returns: The current configuration as a dictionary
        """
        facts, _warnings = Facts(self._module).get_facts(
            self.gather_subset,
            self.gather_network_resources,
            data=data,
        )
        lldp_facts = facts["ansible_network_resources"].get("lldp_global")
        if not lldp_facts:
            return {}
        return lldp_facts

    def execute_module(self):
        """Execute the module

        :rtype: A dictionary
        :returns: The result from module execution
        """
        result = {"changed": False}
        warnings = list()
        commands = list()
        if self.state in self.ACTION_STATES:
            existing_lldp_global_facts = self.get_lldp_global_facts()
        else:
            existing_lldp_global_facts = {}

        if self.state in self.ACTION_STATES or self.state == "rendered":
            commands.extend(self.set_config(existing_lldp_global_facts))

        if commands and self.state in self.ACTION_STATES:
            if not self._module.check_mode:
                self._connection.edit_config(commands)
            result["changed"] = True

        if self.state in self.ACTION_STATES:
            result["commands"] = commands

        if self.state in self.ACTION_STATES or self.state == "gathered":
            changed_lldp_global_facts = self.get_lldp_global_facts()

        elif self.state == "rendered":
            result["rendered"] = commands

        elif self.state == "parsed":
            running_config = self._module.params["running_config"]
            if not running_config:
                self._module.fail_json(
                    msg="value of running_config parameter must not be empty for state parsed",
                )
            result["parsed"] = self.get_lldp_global_facts(data=running_config)

        if self.state in self.ACTION_STATES:
            result["before"] = existing_lldp_global_facts
            if result["changed"]:
                result["after"] = changed_lldp_global_facts
        elif self.state == "gathered":
            result["gathered"] = changed_lldp_global_facts

        result["warnings"] = warnings
        return result

    def set_config(self, existing_lldp_global_facts):
        """Collect the configuration from the args passed to the module,
            collect the current configuration (as a dict from facts)

        :rtype: A list
        :returns: the commands necessary to migrate the current configuration
                  to the desired configuration
        """
        want = self._module.params["config"]
        if not want and self._module.params["state"] == "deleted":
            want = {}
        have = existing_lldp_global_facts
        resp = self.set_state(want, have)
        return to_list(resp)

    def set_state(self, want, have):
        """Select the appropriate function based on the state provided

        :param want: the desired configuration as a dictionary
        :param have: the current configuration as a dictionary
        :rtype: A list
        :returns: the commands necessary to migrate the current configuration
                  to the desired configuration
        """
        commands = []
        state = self._module.params["state"]
        if state in ("merged", "replaced", "rendered") and not want:
            self._module.fail_json(
                msg="value of config parameter must not be empty for state {0}".format(
                    state,
                ),
            )

        if state == "deleted":
            commands = self._state_deleted(want, have)
        elif state in ("merged", "rendered"):
            commands = self._state_merged(want, have)
        elif state in ["replaced", "overridden"]:
            commands = self._state_replaced(want, have)

        return commands

    def _state_replaced(self, want, have):
        """The command generator when state is replaced

        :rtype: A list
        :returns: the commands necessary to migrate the current configuration
                  to the desired configuration
        """
        commands = []

        commands.extend(self._state_deleted(want, have))

        commands.extend(self._state_merged(want, have))

        return commands

    def _state_merged(self, want, have):
        """The command generator when state is merged

        :rtype: A list
        :returns: the commands necessary to merge the provided into
                  the current configuration
        """
        commands = []
        updates = dict_diff(have, want)
        if updates:
            for key, value in iteritems(flatten_dict(remove_empties(updates))):
                commands.append(self._compute_commands(key, value))

        return commands

    def _state_deleted(self, want, have):
        """The command generator when state is deleted

        :rtype: A list
        :returns: the commands necessary to remove the current configuration
                  of the provided objects
        """
        commands = []
        for key, value in iteritems(
            flatten_dict(dict_delete(have, remove_empties(want))),
        ):
            cmd = self._compute_commands(key, value, remove=True)
            if cmd:
                commands.append(cmd)

        return commands

    def _compute_commands(self, key, value=None, remove=False):
        if key in ["holdtime", "reinit", "timer"]:
            cmd = "lldp {0} {1}".format(key, value)
            if remove:
                return "no {0}".format(cmd)
            else:
                return cmd

        elif key == "subinterfaces":
            cmd = "lldp subinterfaces enable"
            if value and not remove:
                return cmd
            elif (not value and not remove) or (value and remove):
                return "no {0}".format(cmd)

        else:
            cmd = "lldp tlv-select {0} disable".format(key.replace("_", "-"))
            if not value and not remove:
                return cmd
            elif (value and not remove) or (not value and remove):
                return "no {0}".format(cmd)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Contributors to the Ansible project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: ec2_vpc_route_table
version_added: 1.0.0
short_description: Manage route tables for AWS Virtual Private Clouds
description:
  - Manage route tables for AWS Virtual Private Clouds (VPCs).
author:
  - Robert Estelle (@erydo)
  - Rob White (@wimnat)
  - Will Thames (@willthames)
options:
  gateway_id:
    description:
      - The ID of the gateway to associate with the route table.
      - If O(gateway_id) is V('None') or V(''), gateway will be disassociated with the route table.
    type: str
    version_added: 3.2.0
  lookup:
    description:
      - Look up route table by either O(tags) or by O(route_table_id).
      - If O(lookup=tag) and O(tags) is not specified then no lookup for an
        existing route table is performed and a new route table will be created.
      - When using O(lookup=tag), multiple matches being found will result in
        a failure and no changes will be made.
      - To change the tags of a route table use O(lookup=id).
      - O(vpc_id) must be specified when O(lookup=tag).
    default: tag
    choices: [ 'tag', 'id' ]
    type: str
  propagating_vgw_ids:
    description: Enable route propagation from virtual gateways specified by ID.
    type: list
    elements: str
  purge_routes:
    description: Purge existing routes that are not found in routes.
    type: bool
    default: True
  purge_subnets:
    description:
      - Purge existing subnets that are not found in subnets.
      - Ignored unless the subnets option is supplied.
    default: True
    type: bool
  route_table_id:
    description:
      - The ID of the route table to update or delete.
      - Required when O(lookup=id).
    type: str
  routes:
    description:
      - List of routes in the route table.
      - Routes are specified as dicts containing the keys V(dest) and one of V(gateway_id),
        V(instance_id), V(network_interface_id), V(transit_gateway_id), or V(vpc_peering_connection_id).
      - The value of V(dest) is used for the destination match. It may be a IPv4 CIDR block
        or a IPv6 CIDR block.
      - If V(gateway_id) is specified, you can refer to the VPC's IGW by using the value V(igw).
      - Routes are required for present states.
    type: list
    elements: dict
    default: []
  state:
    description: Create or destroy the VPC route table.
    default: present
    choices: [ 'present', 'absent' ]
    type: str
  subnets:
    description: An array of subnets to add to this route table. Subnets may be specified
      by either subnet ID, Name tag, or by a CIDR such as '10.0.0.0/24' or 'fd00::/8'.
    type: list
    elements: str
  vpc_id:
    description:
      - VPC ID of the VPC in which to create the route table.
      - Required when O(state=present) or O(lookup=tag).
    type: str
notes:
  - Tags are used to uniquely identify route tables within a VPC when the O(route_table_id) is not supplied.
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.tags
  - amazon.aws.boto3
"""

EXAMPLES = r"""
# Note: These examples do not set authentication details, see the AWS Guide for details.

# Basic creation example:
- name: Set up public subnet route table
  amazon.aws.ec2_vpc_route_table:
    vpc_id: vpc-1245678
    region: us-west-1
    tags:
      Name: Public
    subnets:
      - "{{ jumpbox_subnet.subnet.id }}"
      - "{{ frontend_subnet.subnet.id }}"
      - "{{ vpn_subnet.subnet_id }}"
    routes:
      - dest: 0.0.0.0/0
        gateway_id: "{{ igw.gateway_id }}"
      - dest: ::/0
        gateway_id: "{{ igw.gateway_id }}"
      - dest: 0.0.0.0/0
        gateway_id: "{{ transit_gateway_id }}"
  register: public_route_table

- name: Create VPC gateway
  amazon.aws.ec2_vpc_igw:
    vpc_id: vpc-1245678
  register: vpc_igw

- name: Create gateway route table
  amazon.aws.ec2_vpc_route_table:
    vpc_id: vpc-1245678
    tags:
      Name: Gateway route table
    gateway_id: "{{ vpc_igw.gateway_id }}"
  register: gateway_route_table

- name: Disassociate gateway from route table
  amazon.aws.ec2_vpc_route_table:
    vpc_id: vpc-1245678
    tags:
      Name: Gateway route table
    gateway_id: None
  register: gateway_route_table

- name: Set up NAT-protected route table
  amazon.aws.ec2_vpc_route_table:
    vpc_id: vpc-1245678
    region: us-west-1
    tags:
      Name: Internal
    subnets:
      - "{{ application_subnet.subnet.id }}"
      - 'Database Subnet'
      - '10.0.0.0/8'
    routes:
      - dest: 0.0.0.0/0
        instance_id: "{{ nat.instance_id }}"
  register: nat_route_table

- name: delete route table
  amazon.aws.ec2_vpc_route_table:
    vpc_id: vpc-1245678
    region: us-west-1
    route_table_id: "{{ route_table.id }}"
    lookup: id
    state: absent
"""

RETURN = r"""
route_table:
  description: Route Table result.
  returned: always
  type: complex
  contains:
    associations:
      description: List of associations between the route table and one or more subnets or a gateway.
      returned: always
      type: complex
      contains:
        association_state:
          description: The state of the association.
          returned: always
          type: complex
          contains:
            state:
              description: The state of the association.
              returned: always
              type: str
              sample: associated
            state_message:
              description: Additional information about the state of the association.
              returned: when available
              type: str
              sample: 'Creating association'
        gateway_id:
          description: ID of the internet gateway or virtual private gateway.
          returned: when route table is a gateway route table
          type: str
          sample: igw-03312309
        main:
          description: Whether this is the main route table.
          returned: always
          type: bool
          sample: false
        route_table_association_id:
          description: ID of association between route table and subnet.
          returned: always
          type: str
          sample: rtbassoc-ab47cfc3
        route_table_id:
          description: ID of the route table.
          returned: always
          type: str
          sample: rtb-bf779ed7
        subnet_id:
          description: ID of the subnet.
          returned: when route table is a subnet route table
          type: str
          sample: subnet-82055af9
    id:
      description: ID of the route table (same as RV(route_table.route_table_id) for backwards compatibility).
      returned: always
      type: str
      sample: rtb-bf779ed7
    owner_id:
        description: AWS account owning resource.
        type: str
        sample: 123456789012
    propagating_vgws:
      description: List of Virtual Private Gateways propagating routes.
      returned: always
      type: list
      sample: []
    route_table_id:
      description: ID of the route table.
      returned: always
      type: str
      sample: rtb-bf779ed7
    routes:
      description: List of routes in the route table.
      returned: always
      type: complex
      contains:
        destination_cidr_block:
          description: IPv4 CIDR block of destination
          returned: always
          type: str
          sample: 10.228.228.0/22
        destination_ipv6_cidr_block:
          description: IPv6 CIDR block of destination
          returned: when the route includes an IPv6 destination
          type: str
          sample: 2600:1f1c:1b3:8f00:8000::/65
        gateway_id:
          description: ID of the gateway.
          returned: when gateway is local or internet gateway
          type: str
          sample: local
        instance_id:
          description: ID of a NAT instance.
          returned: when the route is via an EC2 instance
          type: str
          sample: i-abcd123456789
        instance_owner_id:
          description: AWS account owning the NAT instance.
          returned: when the route is via an EC2 instance
          type: str
          sample: 123456789012
        nat_gateway_id:
          description: ID of the NAT gateway.
          returned: when the route is via a NAT gateway
          type: str
          sample: local
        carrier_gateway_id:
          description: ID of the Carrier gateway.
          returned: when the route is via a Carrier gateway
          type: str
          sample: local
          version_added: 6.0.0
        transit_gateway_id:
          description: ID of the Transit gateway.
          returned: when the route is via a Transit gateway
          type: str
          sample: tgw-123456789012
          version_added: 8.3.0
        origin:
          description: mechanism through which the route is in the table.
          returned: always
          type: str
          sample: CreateRouteTable
        state:
          description: state of the route.
          returned: always
          type: str
          sample: active
    tags:
      description: Tags applied to the route table.
      returned: always
      type: dict
      sample:
        Name: Public route table
        Public: 'true'
    vpc_id:
      description: ID for the VPC in which the route lives.
      returned: always
      type: str
      sample: vpc-6e2d2407
"""

import re
from ipaddress import ip_network
from time import sleep
from typing import Any
from typing import Dict
from typing import List
from typing import Optional
from typing import Tuple

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict
from ansible.module_utils.common.dict_transformations import snake_dict_to_camel_dict

from ansible_collections.amazon.aws.plugins.module_utils.ec2 import AnsibleEC2Error
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import associate_route_table
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import create_route
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import create_route_table
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import delete_route
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import delete_route_table
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import describe_ec2_tags
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import describe_internet_gateways
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import describe_route_tables
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import describe_subnets
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import disassociate_route_table
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import enable_vgw_route_propagation
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import ensure_ec2_tags
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import replace_route
from ansible_collections.amazon.aws.plugins.module_utils.modules import AnsibleAWSModule
from ansible_collections.amazon.aws.plugins.module_utils.transformation import ansible_dict_to_boto3_filter_list
from ansible_collections.amazon.aws.plugins.module_utils.waiters import wait_for_resource_state


def find_subnets(
    connection, module: AnsibleAWSModule, vpc_id: str, identified_subnets: Optional[List[str]]
) -> List[Dict[str, Any]]:
    """
    Finds a list of subnets, each identified either by a raw ID, a unique
    'Name' tag, or a CIDR such as 10.0.0.0/8.
    """
    CIDR_RE = re.compile(r"^(\d{1,3}\.){3}\d{1,3}/\d{1,2}$")
    SUBNET_RE = re.compile(r"^subnet-[A-z0-9]+$")

    subnet_ids = []
    subnet_names = []
    subnet_cidrs = []
    for subnet in identified_subnets or []:
        if re.match(SUBNET_RE, subnet):
            subnet_ids.append(subnet)
        elif re.match(CIDR_RE, subnet):
            subnet_cidrs.append(subnet)
        else:
            subnet_names.append(subnet)

    subnets_by_id = []
    if subnet_ids:
        filters = ansible_dict_to_boto3_filter_list({"vpc-id": vpc_id})
        try:
            subnets_by_id = describe_subnets(connection, SubnetIds=subnet_ids, Filters=filters)
        except AnsibleEC2Error as e:
            module.fail_json_aws(e, msg=f"Couldn't find subnet with id {subnet_ids}")

    subnets_by_cidr = []
    if subnet_cidrs:
        filters = ansible_dict_to_boto3_filter_list({"vpc-id": vpc_id, "cidr": subnet_cidrs})
        try:
            subnets_by_cidr = describe_subnets(connection, Filters=filters)
        except AnsibleEC2Error as e:
            module.fail_json_aws(e, msg=f"Couldn't find subnet with cidr {subnet_cidrs}")

    subnets_by_name = []
    if subnet_names:
        filters = ansible_dict_to_boto3_filter_list({"vpc-id": vpc_id, "tag:Name": subnet_names})
        try:
            subnets_by_name = describe_subnets(connection, Filters=filters)
        except AnsibleEC2Error as e:
            module.fail_json_aws(e, msg=f"Couldn't find subnet with names {subnet_names}")

        for name in subnet_names:
            matching_count = len(
                [1 for s in subnets_by_name for t in s.get("Tags", []) if t["Key"] == "Name" and t["Value"] == name]
            )
            if matching_count == 0:
                module.fail_json(msg=f'Subnet named "{name}" does not exist')
            elif matching_count > 1:
                module.fail_json(msg=f'Multiple subnets named "{name}"')

    return subnets_by_id + subnets_by_cidr + subnets_by_name


def find_igw(connection, module: AnsibleAWSModule, vpc_id: str) -> str:
    """
    Finds the Internet gateway for the given VPC ID.
    """
    filters = ansible_dict_to_boto3_filter_list({"attachment.vpc-id": vpc_id})
    igw = describe_internet_gateways(connection, Filters=filters)

    if len(igw) == 0:
        module.fail_json(msg=f"No IGWs found for VPC {vpc_id}")
    elif len(igw) > 1:
        module.fail_json(msg=f"Multiple IGWs found for VPC {vpc_id}")
    return igw[0]["InternetGatewayId"]


def tags_match(match_tags, candidate_tags):
    return all((k in candidate_tags and candidate_tags[k] == v for k, v in match_tags.items()))


def get_route_table_by_id(connection, route_table_id: str) -> Optional[Dict[str, Any]]:
    route_table = None
    route_tables = describe_route_tables(connection, RouteTableIds=[route_table_id])

    if route_tables:
        route_table = route_tables[0]

    return route_table


def get_route_table_by_tags(
    connection, module: AnsibleAWSModule, vpc_id: str, tags: Dict[str, str]
) -> Optional[Dict[str, Any]]:
    count = 0
    route_table = None
    filters = ansible_dict_to_boto3_filter_list({"vpc-id": vpc_id})
    route_tables = describe_route_tables(connection, Filters=filters)

    for table in route_tables:
        this_tags = describe_ec2_tags(connection, module, table["RouteTableId"])
        if tags_match(tags, this_tags):
            route_table = table
            count += 1

    if count > 1:
        module.fail_json(msg="Tags provided do not identify a unique route table")
    return route_table


def route_spec_matches_route(route_spec, route):
    if route_spec.get("GatewayId") and "nat-" in route_spec["GatewayId"]:
        route_spec["NatGatewayId"] = route_spec.pop("GatewayId")
    if route_spec.get("GatewayId") and "vpce-" in route_spec["GatewayId"]:
        if route_spec.get("DestinationCidrBlock", "").startswith("pl-"):
            route_spec["DestinationPrefixListId"] = route_spec.pop("DestinationCidrBlock")

    return set(route_spec.items()).issubset(route.items())


def route_spec_matches_route_cidr(route_spec, route):
    if route_spec.get("DestinationCidrBlock") and route.get("DestinationCidrBlock"):
        return route_spec.get("DestinationCidrBlock") == route.get("DestinationCidrBlock")
    if route_spec.get("DestinationIpv6CidrBlock") and route.get("DestinationIpv6CidrBlock"):
        return route_spec.get("DestinationIpv6CidrBlock") == route.get("DestinationIpv6CidrBlock")
    return False


def rename_key(d, old_key, new_key):
    d[new_key] = d.pop(old_key)


def index_of_matching_route(route_spec, routes_to_match):
    for i, route in enumerate(routes_to_match):
        if route_spec_matches_route(route_spec, route):
            return "exact", i
        elif "Origin" in route and route["Origin"] != "EnableVgwRoutePropagation":  # only replace created routes
            if route_spec_matches_route_cidr(route_spec, route):
                return "replace", i


def ensure_routes(
    connection,
    module: AnsibleAWSModule,
    route_table: Dict[str, Any],
    route_specs: List[Dict[str, Any]],
    purge_routes: bool,
) -> bool:
    routes_to_match = list(route_table["Routes"])
    route_specs_to_create = []
    route_specs_to_recreate = []
    for route_spec in route_specs:
        match = index_of_matching_route(route_spec, routes_to_match)
        if match is None:
            if route_spec.get("DestinationCidrBlock") or route_spec.get("DestinationIpv6CidrBlock"):
                route_specs_to_create.append(route_spec)
            else:
                module.warn(
                    f"Skipping creating {route_spec} because it has no destination cidr block. To add VPC endpoints to"
                    " route tables use the ec2_vpc_endpoint module."
                )
        else:
            if match[0] == "replace":
                if route_spec.get("DestinationCidrBlock"):
                    route_specs_to_recreate.append(route_spec)
                else:
                    module.warn(f"Skipping recreating route {route_spec} because it has no destination cidr block.")
            del routes_to_match[match[1]]

    routes_to_delete = []
    if purge_routes:
        for route in routes_to_match:
            if not route.get("DestinationCidrBlock"):
                module.warn(
                    f"Skipping purging route {route} because it has no destination cidr block. To remove VPC endpoints"
                    " from route tables use the ec2_vpc_endpoint module."
                )
                continue
            if route["Origin"] == "CreateRoute":
                routes_to_delete.append(route)

    if module.check_mode:
        return bool(routes_to_delete or route_specs_to_create or route_specs_to_recreate)

    changed = False
    # Delete routes
    for route in routes_to_delete:
        changed |= delete_route(
            connection,
            route_table_id=route_table["RouteTableId"],
            DestinationCidrBlock=route["DestinationCidrBlock"],
        )
    # Replace routes
    for route_spec in route_specs_to_recreate:
        replace_route(connection, route_table_id=route_table["RouteTableId"], **route_spec)
        changed = True
    # Create routes
    for route_spec in route_specs_to_create:
        create_route(connection, route_table_id=route_table["RouteTableId"], **route_spec)
        changed = True

    return changed


def ensure_subnet_association(
    connection, module: AnsibleAWSModule, vpc_id: str, route_table_id: str, subnet_id: str
) -> Tuple[bool, Optional[str]]:
    filters = ansible_dict_to_boto3_filter_list({"association.subnet-id": subnet_id, "vpc-id": vpc_id})
    route_tables = describe_route_tables(connection, Filters=filters)

    association_id = None
    for route_table in route_tables:
        if route_table.get("RouteTableId"):
            for association in route_table["Associations"]:
                if association["Main"]:
                    continue
                if association["SubnetId"] == subnet_id:
                    if route_table["RouteTableId"] == route_table_id:
                        return False, association["RouteTableAssociationId"]
                    if not module.check_mode:
                        disassociate_route_table(connection, association["RouteTableAssociationId"])
                    return True, association_id

    if not module.check_mode:
        association_id = associate_route_table(connection, route_table_id=route_table_id, SubnetId=subnet_id)[
            "AssociationId"
        ]
    return True, association_id


def ensure_subnet_associations(
    connection,
    module: AnsibleAWSModule,
    route_table: Dict[str, Any],
    subnets: List[Dict[str, Any]],
    purge_subnets: bool,
) -> bool:
    current_association_ids = [
        association["RouteTableAssociationId"]
        for association in route_table["Associations"]
        if not association["Main"] and association.get("SubnetId")
    ]
    new_association_ids = []
    changed = False
    for subnet in subnets:
        result, association_id = ensure_subnet_association(
            connection=connection,
            module=module,
            vpc_id=route_table["VpcId"],
            route_table_id=route_table["RouteTableId"],
            subnet_id=subnet["SubnetId"],
        )
        changed |= result
        if changed and module.check_mode:
            return True
        new_association_ids.append(association_id)

    if purge_subnets:
        to_delete = [
            association_id for association_id in current_association_ids if association_id not in new_association_ids
        ]
        for association_id in to_delete:
            changed = True
            if not module.check_mode:
                disassociate_route_table(connection, association_id)

    return changed


def disassociate_gateway(connection, module: AnsibleAWSModule, route_table: Dict[str, Any]) -> bool:
    # Delete all gateway associations that have state = associated
    # Subnet associations are handled in its method
    changed = False
    associations_to_delete = [
        association["RouteTableAssociationId"]
        for association in route_table["Associations"]
        if not association["Main"]
        and association.get("GatewayId")
        and association["AssociationState"]["State"] in ["associated", "associating"]
    ]
    for association_id in associations_to_delete:
        changed = True
        if not module.check_mode:
            disassociate_route_table(connection, association_id)

    return changed


def associate_gateway(connection, module: AnsibleAWSModule, route_table: Dict[str, Any], gateway_id: str) -> bool:
    filters = ansible_dict_to_boto3_filter_list({"association.gateway-id": gateway_id, "vpc-id": route_table["VpcId"]})
    route_tables = describe_route_tables(connection, Filters=filters)

    for table in route_tables:
        if table.get("RouteTableId"):
            for association in table.get("Associations"):
                if association["Main"]:
                    continue
                if association.get("GatewayId", "") == gateway_id and (
                    association["AssociationState"]["State"] in ["associated", "associating"]
                ):
                    if table["RouteTableId"] == route_table["RouteTableId"]:
                        return False
                    elif module.check_mode:
                        return True
                    else:
                        disassociate_route_table(connection, association["RouteTableAssociationId"])

    if not module.check_mode:
        associate_route_table(connection, route_table_id=route_table["RouteTableId"], GatewayId=gateway_id)
    return True


def ensure_propagation(
    connection, module: AnsibleAWSModule, route_table: Dict[str, Any], propagating_vgw_ids: List[str]
) -> bool:
    changed = False
    gateways = [gateway["GatewayId"] for gateway in route_table["PropagatingVgws"]]
    vgws_to_add = set(propagating_vgw_ids) - set(gateways)
    if not vgws_to_add:
        return changed
    changed = True
    if module.check_mode:
        return changed
    for vgw_id in vgws_to_add:
        enable_vgw_route_propagation(connection, route_table_id=route_table["RouteTableId"], gateway_id=vgw_id)

    return changed


def ensure_route_table_absent(connection, module: AnsibleAWSModule) -> Dict[str, bool]:
    lookup = module.params.get("lookup")
    route_table_id = module.params.get("route_table_id")
    tags = module.params.get("tags")
    vpc_id = module.params.get("vpc_id")
    purge_subnets = module.params.get("purge_subnets")
    route_table = None

    if lookup == "tag" and tags is not None:
        route_table = get_route_table_by_tags(connection, module, vpc_id, tags)
    elif lookup == "id":
        route_table = get_route_table_by_id(connection, route_table_id)

    if route_table is None:
        return {"changed": False}

    # disassociate subnets and gateway before deleting route table
    changed = False
    if module.check_mode:
        changed = True
    else:
        changed = ensure_subnet_associations(
            connection=connection, module=module, route_table=route_table, subnets=[], purge_subnets=purge_subnets
        )
        changed |= disassociate_gateway(connection=connection, module=module, route_table=route_table)
        changed |= delete_route_table(connection, route_table_id=route_table["RouteTableId"])

    return {"changed": changed}


def get_route_table_info(connection, module: AnsibleAWSModule, route_table: Dict[str, Any]) -> Optional[Dict[str, Any]]:
    result = get_route_table_by_id(connection, route_table["RouteTableId"]) or {}
    if result:
        result["Tags"] = describe_ec2_tags(connection, module, route_table["RouteTableId"])
        result = camel_dict_to_snake_dict(result, ignore_list=["Tags"])
        # backwards compatibility
        result["id"] = result["route_table_id"]
    return result


def create_route_spec(connection, module, vpc_id):
    routes = module.params.get("routes")
    for route_spec in routes:
        cidr_block_type = str(type(ip_network(route_spec["dest"])))
        if "IPv4" in cidr_block_type:
            rename_key(route_spec, "dest", "destination_cidr_block")
        if "IPv6" in cidr_block_type:
            rename_key(route_spec, "dest", "destination_ipv6_cidr_block")

        if route_spec.get("gateway_id") and route_spec["gateway_id"].lower() == "igw":
            igw = find_igw(connection, module, vpc_id)
            route_spec["gateway_id"] = igw
        if route_spec.get("gateway_id") and route_spec["gateway_id"].startswith("nat-"):
            rename_key(route_spec, "gateway_id", "nat_gateway_id")
        if route_spec.get("gateway_id") and route_spec["gateway_id"].startswith("cagw-"):
            rename_key(route_spec, "gateway_id", "carrier_gateway_id")
        if route_spec.get("gateway_id") and route_spec["gateway_id"].startswith("tgw-"):
            rename_key(route_spec, "gateway_id", "transit_gateway_id")

    return snake_dict_to_camel_dict(routes, capitalize_first=True)


def ensure_route_table_present(connection, module: AnsibleAWSModule) -> Dict[str, Any]:
    gateway_id = module.params.get("gateway_id")
    lookup = module.params.get("lookup")
    propagating_vgw_ids = module.params.get("propagating_vgw_ids")
    purge_routes = module.params.get("purge_routes")
    purge_subnets = module.params.get("purge_subnets")
    purge_tags = module.params.get("purge_tags")
    route_table_id = module.params.get("route_table_id")
    subnets = module.params.get("subnets")
    tags = module.params.get("tags")
    vpc_id = module.params.get("vpc_id")
    routes = create_route_spec(connection, module, vpc_id)

    changed = False
    tags_valid = False
    route_table = None

    if lookup == "tag" and tags is not None:
        route_table = get_route_table_by_tags(connection, module, vpc_id, tags)
    elif lookup == "id":
        route_table = get_route_table_by_id(connection, route_table_id)

    # If no route table returned then create new route table
    if route_table is None:
        changed = True
        if module.check_mode:
            route_table = {"id": "rtb-xxxxxxxx", "route_table_id": "rtb-xxxxxxxx", "vpc_id": vpc_id}
            module.exit_json(changed=changed, route_table=route_table)

        route_table = create_route_table(connection, vpc_id, tags)
        # try to wait for route table to be present before moving on
        wait_for_resource_state(connection, module, "route_table_exists", RouteTableIds=[route_table["RouteTableId"]])

    if routes is not None:
        changed |= ensure_routes(
            connection=connection, module=module, route_table=route_table, route_specs=routes, purge_routes=purge_routes
        )

    if propagating_vgw_ids is not None:
        changed |= ensure_propagation(
            connection=connection, module=module, route_table=route_table, propagating_vgw_ids=propagating_vgw_ids
        )

    if not tags_valid and tags is not None:
        changed |= ensure_ec2_tags(
            connection,
            module,
            route_table["RouteTableId"],
            tags=tags,
            purge_tags=purge_tags,
            retry_codes=["InvalidRouteTableID.NotFound"],
        )
        route_table["Tags"] = describe_ec2_tags(connection, module, route_table["RouteTableId"])

    if subnets is not None:
        associated_subnets = find_subnets(connection, module, vpc_id, subnets)
        changed |= ensure_subnet_associations(
            connection=connection,
            module=module,
            route_table=route_table,
            subnets=associated_subnets,
            purge_subnets=purge_subnets,
        )

    if gateway_id == "None" or gateway_id == "":
        gateway_changed = disassociate_gateway(connection=connection, module=module, route_table=route_table)
    elif gateway_id is not None:
        gateway_changed = associate_gateway(
            connection=connection, module=module, route_table=route_table, gateway_id=gateway_id
        )
    else:
        gateway_changed = False
    changed = changed or gateway_changed

    if changed:
        # pause to allow route table routes/subnets/associations to be updated before exiting with final state
        sleep(5)

    return dict(changed=changed, route_table=get_route_table_info(connection, module, route_table))


def main() -> None:
    argument_spec = dict(
        gateway_id=dict(type="str"),
        lookup=dict(default="tag", choices=["tag", "id"]),
        propagating_vgw_ids=dict(type="list", elements="str"),
        purge_routes=dict(default=True, type="bool"),
        purge_subnets=dict(default=True, type="bool"),
        purge_tags=dict(type="bool", default=True),
        route_table_id=dict(),
        routes=dict(default=[], type="list", elements="dict"),
        state=dict(default="present", choices=["present", "absent"]),
        subnets=dict(type="list", elements="str"),
        tags=dict(type="dict", aliases=["resource_tags"]),
        vpc_id=dict(),
    )

    module = AnsibleAWSModule(
        argument_spec=argument_spec,
        required_if=[
            ["lookup", "id", ["route_table_id"]],
            ["lookup", "tag", ["vpc_id"]],
            ["state", "present", ["vpc_id"]],
        ],
        supports_check_mode=True,
    )

    connection = module.client("ec2")
    method_operation = (
        ensure_route_table_present if module.params.get("state") == "present" else ensure_route_table_absent
    )

    try:
        result = method_operation(connection, module)
    except AnsibleEC2Error as e:
        module.fail_json_aws_error(e)

    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2023, Tim Cragg (@timcragg) <tcragg@cisco.com>
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "certified"}

DOCUMENTATION = r"""
---
module: aci_config_export_policy
short_description: Manage Configuration Export Policy (config:ExportP)
description:
- Manage Configuration Export Policies on Cisco ACI fabrics.
options:
  name:
    description:
    - The name of the Configuration Export Policy.
    type: str
  description:
    description:
    - The description of the Configuration Export Policy.
    type: str
  format:
    description:
    - The format of the export file.
    - This defaults to json on the APIC when unset on creation.
    type: str
    choices: [ json, xml ]
  target_dn:
    description:
    - The distinguished name of the object to be exported.
    - If no target_dn is included, the APIC will export the policy universe.
    type: str
  snapshot:
    description:
    - Enables a snapshot of the configuration export policy.
    - This defaults to False on the APIC when unset on creation.
    type: bool
  export_destination:
    description:
    - The name of the remote path policy used for storing the generated configuration backup data for the configuration export policy.
    type: str
  scheduler:
    description:
    - The name of the scheduler policy used for running scheduled export jobs.
    type: str
  start_now:
    description:
    - Specifies if the configuration export policy should be applied now or at another time.
    - This defaults to False on the APIC when unset on creation.
    type: bool
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    type: str
    choices: [ absent, present, query ]
    default: present
extends_documentation_fragment:
- cisco.aci.aci
- cisco.aci.annotation

seealso:
- name: APIC Management Information Model reference
  description: More information about the internal APIC class B(config:ExportP).
  link: https://developer.cisco.com/docs/apic-mim-ref/

author:
- Tim Cragg (@timcragg)
"""

EXAMPLES = r"""
- name: Add a Configuration Export Policy
  cisco.aci.aci_config_export_policy:
    host: apic
    username: admin
    password: SomeSecretPassword
    name: ans_conf_export
    state: present
  delegate_to: localhost

- name: Query a Configuration Export Policy
  cisco.aci.aci_config_export_policy:
    host: apic
    username: admin
    password: SomeSecretPassword
    name: ans_conf_export
    state: query
  delegate_to: localhost

- name: Query all Configuration Export Policies
  cisco.aci.aci_config_export_policy:
    host: apic
    username: admin
    password: SomeSecretPassword
    state: query
  delegate_to: localhost

- name: Remove a Configuration Export Policies
  cisco.aci.aci_file_remote_path:
    host: apic
    username: admin
    password: SomeSecretPassword
    name: ans_conf_export
    state: absent
  delegate_to: localhost
"""

RETURN = r"""
current:
  description: The existing configuration from the APIC after the module has finished
  returned: success
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production environment",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
error:
  description: The error information as returned from the APIC
  returned: failure
  type: dict
  sample:
    {
        "code": "122",
        "text": "unknown managed object class foo"
    }
raw:
  description: The raw output returned by the APIC REST API (xml or json)
  returned: parse error
  type: str
  sample: '<?xml version="1.0" encoding="UTF-8"?><imdata totalCount="1"><error code="122" text="unknown managed object class foo"/></imdata>'
sent:
  description: The actual/minimal configuration pushed to the APIC
  returned: info
  type: list
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment"
            }
        }
    }
previous:
  description: The original configuration from the APIC before the module has started
  returned: info
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
proposed:
  description: The assembled configuration from the user-provided parameters
  returned: info
  type: dict
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment",
                "name": "production"
            }
        }
    }
filter_string:
  description: The filter string used for the request
  returned: failure or debug
  type: str
  sample: ?rsp-prop-include=config-only
method:
  description: The HTTP method used for the request to the APIC
  returned: failure or debug
  type: str
  sample: POST
response:
  description: The HTTP response from the APIC
  returned: failure or debug
  type: str
  sample: OK (30 bytes)
status:
  description: The HTTP status from the APIC
  returned: failure or debug
  type: int
  sample: 200
url:
  description: The HTTP url used for the request to the APIC
  returned: failure or debug
  type: str
  sample: https://10.11.12.13/api/mo/uni/tn-production.json
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.aci.plugins.module_utils.aci import ACIModule, aci_argument_spec, aci_annotation_spec


def main():
    argument_spec = aci_argument_spec()
    argument_spec.update(aci_annotation_spec())
    argument_spec.update(
        name=dict(type="str"),
        description=dict(type="str"),
        format=dict(type="str", choices=["json", "xml"]),
        target_dn=dict(type="str"),
        snapshot=dict(type="bool"),
        export_destination=dict(type="str"),
        scheduler=dict(type="str"),
        start_now=dict(type="bool"),
        state=dict(type="str", default="present", choices=["absent", "present", "query"]),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "present", ["name"]],
            ["state", "absent", ["name"]],
        ],
    )

    aci = ACIModule(module)

    name = module.params.get("name")
    description = module.params.get("description")
    format = module.params.get("format")
    target_dn = module.params.get("target_dn")
    snapshot = aci.boolean(module.params.get("snapshot"))
    export_destination = module.params.get("export_destination")
    scheduler = module.params.get("scheduler")
    start_now = aci.boolean(module.params.get("start_now"), "triggered", "untriggered")
    state = module.params.get("state")

    aci.construct_url(
        root_class=dict(
            aci_class="configExportP",
            aci_rn="fabric/configexp-{0}".format(name),
            module_object=name,
            target_filter={"name": name},
        ),
        child_classes=["configRsExportScheduler", "configRsRemotePath"],
    )
    aci.get_existing()

    if state == "present":
        child_configs = []
        if scheduler is not None:
            child_configs.append(
                dict(
                    configRsExportScheduler=dict(
                        attributes=dict(tnTrigSchedPName=scheduler),
                    )
                )
            )
        if export_destination is not None:
            child_configs.append(
                dict(
                    configRsRemotePath=dict(
                        attributes=dict(tnFileRemotePathName=export_destination),
                    )
                )
            )
        aci.payload(
            aci_class="configExportP",
            class_config=dict(
                name=name,
                descr=description,
                format=format,
                targetDn=target_dn,
                snapshot=snapshot,
                adminSt=start_now,
            ),
            child_configs=child_configs,
        )

        aci.get_diff(aci_class="configExportP")

        aci.post_config()

    elif state == "absent":
        aci.delete_config()

    aci.exit_json()


if __name__ == "__main__":
    main()



#!/usr/bin/python
# coding: utf-8
# Author: Mario Lenz <m@riolenz.de>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later
# SPDX-FileCopyrightText: Ansible Project, 2022
"""The ansible-community CLI program."""

import argparse


def main():
    '''Main entrypoint for the ansible-community CLI program.'''
    parser = argparse.ArgumentParser()
    parser.add_argument(
        '--version',
        action='version',
        version='Ansible community version 11.1.0',
        help="show the version of the Ansible community package",
    )
    parser.parse_args()
    parser.print_help()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

"""
The module file for eos_logging_global
"""

from __future__ import absolute_import, division, print_function


__metaclass__ = type

DOCUMENTATION = """
---
module: eos_logging_global
short_description: Manages logging resource module
description: This module configures and manages the attributes of  logging on Arista
  EOS platforms.
version_added: 3.0.0
author: Gomathi Selvi Srinivasan (@GomathiselviS)
notes:
- Tested against Arista EOS 4.24.6M
- This module works with connection C(network_cli). See the L(EOS Platform Options,eos_platform_options).
options:
   config:
      description: A dictionary of logging options
      type: dict
      suboptions:
        buffered:
          description:
          - Set buffered logging parameters.
          type: dict
          suboptions: &message_options
              severity: &sev
                description: Severity level .
                type: str
                choices:
                - emergencies
                - alerts
                - critical
                - errors
                - warnings
                - notifications
                - informational
                - debugging
              buffer_size:
                description: Logging buffer size
                type: int
        console:
          description:
          - Set console logging parameters.
          type: dict
          suboptions:
              severity: *sev
        event:
          description: Global events
          type: str
          choices: ["link-status", "port-channel", "spanning-tree"]
        facility:
          description: Set logging facility.
          type: str
          "choices": [
                        "auth",
                        "cron",
                        "daemon",
                        "kern",
                        "local0",
                        "local1",
                        "local2",
                        "local3",
                        "local4",
                        "local5",
                        "local6",
                        "local7",
                        "lpr",
                        "mail",
                        "news",
                        "sys10",
                        "sys11",
                        "sys12",
                        "sys13",
                        "sys14",
                        "sys9",
                        "syslog",
                        "user",
                        "uucp",
                    ]
        format:
          description: Set logging format parameters
          type: dict
          suboptions:
            hostname:
              description: Specify hostname logging format.
              type: str
            timestamp:
              description: Set timestamp logging parameters.
              type: dict
              suboptions:
                high_resolution:
                  description: RFC3339 timestamps.
                  type: bool
                traditional:
                  description: Traditional syslog timestamp format as specified in RFC3164.
                  type: dict
                  suboptions:
                    state:
                      description: When enabled traditional timestamp format is set.
                      type: str
                      choices: ["enabled", "disabled"]
                    timezone:
                      description: Show timezone in traditional format timestamp
                      type: bool
                    year:
                      description: Show year in traditional format timestamp
                      type: bool
            sequence_numbers:
              description:  No. of log messages.
              type: bool
        hosts: &host
          description: Set syslog server IP address and parameters.
          type: list
          elements: dict
          suboptions:
            name:
              description: Hostname or IP address of the syslog server.
              type: str
            add:
              description: Configure ports on the given host.
              type: bool
            remove:
              description: Remove configured ports from the given host
              type: bool
            protocol:
              description: Set syslog server transport protocol
              type: str
              choices: ["tcp", "udp"]
            port:
              description: Port of the syslog server.
              type: int
        level:
          description: Configure logging severity
          type: dict
          suboptions:
            facility:
              description: Facility level
              type: str
            severity: *sev
        monitor:
          description: Set terminal monitor severity
          type: str
        turn_on:
          description: Turn on logging.
          type: bool
        persistent:
          description: Save logging messages to the flash disk.
          type: dict
          suboptions:
            set:
              description: Save logging messages to the flash dis.
              type: bool
            size:
              description: The maximum size (in bytes) of logging file stored on flash disk.
              type: int
        policy:
          description: Configure logging policies.
          type: dict
          suboptions:
            invert_result:
              description: Invert the match of match-list.
              type: bool
            match_list:
              description: Configure logging message filtering.
              type: str
        qos:
          description: Set DSCP value in IP header.
          type: int
        relogging_interval:
          description: Configure relogging-interval for critical log messages
          type: int
        repeat_messages:
          description: Repeat messages instead of summarizing number of repeats
          type: bool
        source_interface: &srcint
          description: Use IP Address of interface as source IP of log messages.
          type: str
        synchronous:
          description: Set synchronizing unsolicited with solicited messages
          type: dict
          suboptions:
            set:
              description: Set synchronizing unsolicited with solicited messages.
              type: bool
            level:
              description: Configure logging severity
              type: str
        trap:
          description: Severity of messages sent to the syslog server.
          type: dict
          suboptions:
            set:
              description: Severity of messages sent to the syslog server.
              type: bool
            severity: *sev
        vrfs:
          description: Specify vrf
          type: list
          elements: dict
          suboptions:
            name:
              description: vrf name.
              type: str
            hosts: *host
            source_interface: *srcint
   running_config:
      description:
      - This option is used only with state I(parsed).
      - The value of this option should be the output received from the EOS device by
        executing the command B(show running-config | section access-list).
      - The states I(replaced) and I(overridden) have identical
        behaviour for this module.
      - The state I(parsed) reads the configuration from C(running_config) option and
        transforms it into Ansible structured data as per the resource module's argspec
        and the value is then returned in the I(parsed) key within the result.
      type: str
   state:
      description:
      - The state the configuration should be left in.
      type: str
      choices:
      - deleted
      - merged
      - overridden
      - replaced
      - gathered
      - rendered
      - parsed
      default: merged
"""
EXAMPLES = """
# Using merged
# Before state

# test(config)#show running-config | section logging
# test(config)#

- name: Merge provided configuration with device configuration
  arista.eos.eos_logging_global:
    config:
      hosts:
        - name: "host01"
          protocol: "tcp"
        - name: "11.11.11.1"
          port: 25
      vrfs:
        - name: "vrf01"
          source_interface: "Ethernet1"
        - name: "vrf02"
          hosts:
            - name: "hostvrf1"
              protocol: "tcp"
            - name: "24.1.1.1"
              port: "33"

# After State:

# test(config)#show running-config | section logging
# logging host 11.11.11.1 25
# logging host host01 514 protocol tcp
# logging vrf vrf02 host 24.1.1.1 33
# logging vrf vrf02 host hostvrf1 514 protocol tcp
# logging vrf vrf01 source-interface Ethernet1
# test(config)#
#
#
# Module Execution:
# "after": {
#         "hosts": [
#             {
#                 "name": "11.11.11.1",
#                 "port": 25
#             },
#             {
#                 "name": "host01",
#                 "port": 514,
#                 "protocol": "tcp"
#             }
#         ],
#         "vrfs": [
#             {
#                 "name": "vrf01",
#                 "source_interface": "Ethernet1"
#             },
#             {
#                 "hosts": [
#                     {
#                         "name": "24.1.1.1",
#                         "port": 33
#                     },
#                     {
#                         "name": "hostvrf1",
#                         "port": 514,
#                         "protocol": "tcp"
#                     }
#                 ],
#                 "name": "vrf02"
#             }
#         ]
#     },
#     "before": {},
#     "changed": true,
#     "commands": [
#         "logging host host01 protocol tcp",
#         "logging host 11.11.11.1 25",
#         "logging vrf vrf01 source-interface Ethernet1",
#         "logging vrf vrf02 host hostvrf1 protocol tcp",
#         "logging vrf vrf02 host 24.1.1.1 33"
#     ],
#

# Using replaced:
# Before State:

# test(config)#show running-config | section logging
# logging host 11.11.11.1 25
# logging host host01 514 protocol tcp
# logging vrf vrf02 host 24.1.1.1 33
# logging vrf vrf02 host hostvrf1 514 protocol tcp
# logging format timestamp traditional timezone
# logging vrf vrf01 source-interface Ethernet1
# logging policy match inverse-result match-list                           list01 discard
# logging persistent 4096
# !
# logging level AAA alerts
# test(config)#

- name: Repalce
  arista.eos.eos_logging_global:
    config:
      synchronous:
        set: true
      trap:
        severity: "critical"
      hosts:
        - name: "host02"
          protocol: "tcp"
      vrfs:
        - name: "vrf03"
          source_interface: "Vlan100"
        - name: "vrf04"
          hosts:
            - name: "hostvrf1"
              protocol: "tcp"

    state: replaced

# After State:
# test(config)#show running-config | section logging
# logging synchronous
# logging trap critical
# logging host host02 514 protocol tcp
# logging vrf vrf04 host hostvrf1 514 protocol tcp
# logging vrf vrf03 source-interface Vlan100
# test(config)#
#
# Module Execution:
# "after": {
#         "hosts": [
#             {
#                 "name": "host02",
#                 "port": 514,
#                 "protocol": "tcp"
#             }
#         ],
#         "synchronous": {
#             "set": true
#         },
#         "trap": {
#            "severity": "critical"
#         },
#         "vrfs": [
#             {
#                 "name": "vrf03",
#                 "source_interface": "Vlan100"
#             },
#             {
#                 "hosts": [
#                     {
#                         "name": "hostvrf1",
#                         "port": 514,
#                         "protocol": "tcp"
#                     }
#                 ],
#                 "name": "vrf04"
#             }
#         ]
#     },
#     "before": {
#         "format": {
#             "timestamp": {
#                 "traditional": {
#                     "timezone": true
#                 }
#             }
#         },
#         "hosts": [
#             {
#                 "name": "11.11.11.1",
#                 "port": 25
#             },
#             {
#                 "name": "host01",
#                 "port": 514,
#                 "protocol": "tcp"
#             }
#         ],
#         "level": {
#             "facility": "AAA",
#             "severity": "alerts"
#         },
#         "persistent": {
#             "size": 4096
#         },
#         "policy": {
#             "invert_result": true,
#             "match_list": "list01"
#         },
#         "vrfs": [
#             {
#                 "name": "vrf01",
#                 "source_interface": "Ethernet1"
#             },
#             {
#                 "hosts": [
#                     {
#                         "name": "24.1.1.1",
#                         "port": 33
#                     },
#                     {
#                         "name": "hostvrf1",
#                         "port": 514,
#                         "protocol": "tcp"
#                     }
#                 ],
#                 "name": "vrf02"
#             }
#         ]
#     },
#     "changed": true,
#     "commands": [
#         "logging host host02 protocol tcp",
#         "no logging host 11.11.11.1 25",
#         "no logging host host01 514 protocol tcp",
#         "logging vrf vrf03 source-interface Vlan100",
#         "logging vrf vrf04 host hostvrf1 protocol tcp",
#         "no logging vrf vrf01 source-interface Ethernet1",
#         "no logging vrf vrf02 host 24.1.1.1 33",
#         "no logging vrf vrf02 host hostvrf1 514 protocol tcp",
#         "no logging format timestamp traditional timezone",
#         "no logging level AAA alerts",
#         "no logging persistent 4096",
#         "no logging policy match invert-result match-list list01 discard",
#         "logging synchronous",
#         "logging trap critical"
#     ],
#
#


# Using overridden:
# Before State:

# test(config)#show running-config | section logging
# logging host 11.11.11.1 25
# logging host host01 514 protocol tcp
# logging vrf vrf02 host 24.1.1.1 33
# logging vrf vrf02 host hostvrf1 514 protocol tcp
# logging format timestamp traditional timezone
# logging vrf vrf01 source-interface Ethernet1
# logging policy match inverse-result match-list                           list01 discard
# logging persistent 4096
# !
# logging level AAA alerts
# test(config)#

- name: Repalce
  arista.eos.eos_logging_global:
    config:
      synchronous:
        set: true
      trap:
        severity: "critical"
      hosts:
        - name: "host02"
          protocol: "tcp"
      vrfs:
        - name: "vrf03"
          source_interface: "Vlan100"
        - name: "vrf04"
          hosts:
            - name: "hostvrf1"
              protocol: "tcp"

    state: overridden

# After State:
# test(config)#show running-config | section logging
# logging synchronous
# logging trap critical
# logging host host02 514 protocol tcp
# logging vrf vrf04 host hostvrf1 514 protocol tcp
# logging vrf vrf03 source-interface Vlan100
# test(config)#
#
# Module Execution:
# "after": {
#         "hosts": [
#             {
#                 "name": "host02",
#                 "port": 514,
#                 "protocol": "tcp"
#             }
#         ],
#         "synchronous": {
#             "set": true
#         },
#         "trap": {
#            "severity": "critical"
#         },
#         "vrfs": [
#             {
#                 "name": "vrf03",
#                 "source_interface": "Vlan100"
#             },
#             {
#                 "hosts": [
#                     {
#                         "name": "hostvrf1",
#                         "port": 514,
#                         "protocol": "tcp"
#                     }
#                 ],
#                 "name": "vrf04"
#             }
#         ]
#     },
#     "before": {
#         "format": {
#             "timestamp": {
#                 "traditional": {
#                     "timezone": true
#                 }
#             }
#         },
#         "hosts": [
#             {
#                 "name": "11.11.11.1",
#                 "port": 25
#             },
#             {
#                 "name": "host01",
#                 "port": 514,
#                 "protocol": "tcp"
#             }
#         ],
#         "level": {
#             "facility": "AAA",
#             "severity": "alerts"
#         },
#         "persistent": {
#             "size": 4096
#         },
#         "policy": {
#             "invert_result": true,
#             "match_list": "list01"
#         },
#         "vrfs": [
#             {
#                 "name": "vrf01",
#                 "source_interface": "Ethernet1"
#             },
#             {
#                 "hosts": [
#                     {
#                         "name": "24.1.1.1",
#                         "port": 33
#                     },
#                     {
#                         "name": "hostvrf1",
#                         "port": 514,
#                         "protocol": "tcp"
#                     }
#                 ],
#                 "name": "vrf02"
#             }
#         ]
#     },
#     "changed": true,
#     "commands": [
#         "logging host host02 protocol tcp",
#         "no logging host 11.11.11.1 25",
#         "no logging host host01 514 protocol tcp",
#         "logging vrf vrf03 source-interface Vlan100",
#         "logging vrf vrf04 host hostvrf1 protocol tcp",
#         "no logging vrf vrf01 source-interface Ethernet1",
#         "no logging vrf vrf02 host 24.1.1.1 33",
#         "no logging vrf vrf02 host hostvrf1 514 protocol tcp",
#         "no logging format timestamp traditional timezone",
#         "no logging level AAA alerts",
#         "no logging persistent 4096",
#         "no logging policy match invert-result match-list list01 discard",
#         "logging synchronous",
#         "logging trap critical"
#     ],
#
#

# Using deleted:

# Before State:
# test(config)#show running-config | section logging
# logging synchronous level critical
# logging host 11.11.11.1 25
# logging host host01 514 protocol tcp
# logging host host02 514 protocol tcp
# logging vrf vrf02 host 24.1.1.1 33
# logging vrf vrf02 host hostvrf1 514 protocol tcp
# logging vrf vrf04 host hostvrf1 514 protocol tcp
# logging vrf vrf01 source-interface Ethernet1
# logging vrf vrf03 source-interface Vlan100
# test(config)#

- name: Delete all logging configs
  arista.eos.eos_logging_global:
    state: deleted
  become: true

# After state:
# test(config)#show running-config | section logging
# test(config)#
#
# "after": {},
#     "before": {
#         "hosts": [
#             {
#                 "name": "11.11.11.1",
#                 "port": 25
#             },
#             {
#                 "name": "host01",
#                 "port": 514,
#                 "protocol": "tcp"
#             },
#             {
#                 "name": "host02",
#                 "port": 514,
#                 "protocol": "tcp"
#             }
#         ],
#         "synchronous": {
#             "level": "critical"
#         },
#         "vrfs": [
#             {
#                 "name": "vrf01",
#                 "source_interface": "Ethernet1"
#             },
#             {
#                 "hosts": [
#                     {
#                         "name": "24.1.1.1",
#                         "port": 33
#                     },
#                     {
#                         "name": "hostvrf1",
#                         "port": 514,
#                         "protocol": "tcp"
#                     }
#                 ],
#                 "name": "vrf02"
#             },
#             {
#                 "name": "vrf03",
#                 "source_interface": "Vlan100"
#             },
#             {
#                 "hosts": [
#                     {
#                         "name": "hostvrf1",
#                         "port": 514,
#                         "protocol": "tcp"
#                     }
#                 ],
#                 "name": "vrf04"
#             }
#         ]
#     },
#     "changed": true,
#     "commands": [
#         "no logging host 11.11.11.1 25",
#         "no logging host host01 514 protocol tcp",
#         "no logging host host02 514 protocol tcp",
#         "no logging vrf vrf01 source-interface Ethernet1",
#         "no logging vrf vrf02 host 24.1.1.1 33",
#         "no logging vrf vrf02 host hostvrf1 514 protocol tcp",
#         "no logging vrf vrf03 source-interface Vlan100",
#         "no logging vrf vrf04 host hostvrf1 514 protocol tcp",
#         "no logging synchronous level critical"
#     ],

# Using parsed:
# parsed.cfg

# logging host 11.11.11.1 25
# logging host host01 514 protocol tcp
# logging vrf vrf02 host 24.1.1.1 33
# logging vrf vrf02 host hostvrf1 514 protocol tcp
# logging format timestamp traditional timezone
# logging vrf vrf01 source-interface Ethernet1
# logging policy match inverse-result match-list                           list01 discard
# logging persistent 4096
# !
# logging level AAA alerts

- name: parse configs
  arista.eos.eos_logging_global:
    running_config: "{{ lookup('file', './parsed.cfg') }}"
    state: parsed

# Module Execution
# "parsed": {
#         "format": {
#             "timestamp": {
#                 "traditional": {
#                     "timezone": true
#                 }
#             }
#         },
#         "hosts": [
#             {
#                 "name": "11.11.11.1",
#                 "port": 25
#             },
#             {
#                 "name": "host01",
#                 "port": 514,
#                 "protocol": "tcp"
#             }
#         ],
#         "level": {
#             "facility": "AAA",
#             "severity": "alerts"
#         },
#         "persistent": {
#             "size": 4096
#         },
#         "policy": {
#             "invert_result": true,
#             "match_list": "list01"
#         },
#         "vrfs": [
#             {
#                 "name": "vrf01",
#                 "source_interface": "Ethernet1"
#             },
#             {
#                 "hosts": [
#                     {
#                         "name": "24.1.1.1",
#                         "port": 33
#                     },
#                     {
#                         "name": "hostvrf1",
#                         "port": 514,
#                         "protocol": "tcp"
#                     }
#                 ],
#                 "name": "vrf02"
#             }
#         ]
#     }
#

# Using gathered:
# Before State:
# test(config)#show running-config | section logging
# logging host 11.11.11.1 25
# logging host host01 514 protocol tcp
# logging vrf vrf02 host 24.1.1.1 33
# logging vrf vrf02 host hostvrf1 514 protocol tcp
# logging format timestamp traditional timezone
# logging vrf vrf01 source-interface Ethernet1
# logging policy match inverse-result match-list                           list01 discard
# logging persistent 4096
# !
# logging level AAA alerts
# test(config)#

- name: gather configs
  arista.eos.eos_logging_global:
    state: gathered

# Module Execution:
# "gathered": {
#         "format": {
#             "timestamp": {
#                 "traditional": {
#                     "timezone": true
#                 }
#             }
#         },
#         "hosts": [
#             {
#                 "name": "11.11.11.1",
#                 "port": 25
#             },
#             {
#                 "name": "host01",
#                 "port": 514,
#                 "protocol": "tcp"
#             }
#         ],
#         "level": {
#             "facility": "AAA",
#             "severity": "alerts"
#         },
#         "persistent": {
#             "size": 4096
#         },
#         "policy": {
#             "invert_result": true,
#             "match_list": "list01"
#         },
#         "vrfs": [
#             {
#                 "name": "vrf01",
#                 "source_interface": "Ethernet1"
#             },
#             {
#                 "hosts": [
#                     {
#                         "name": "24.1.1.1",
#                         "port": 33
#                     },
#                     {
#                         "name": "hostvrf1",
#                         "port": 514,
#                         "protocol": "tcp"
#                     }
#                 ],
#                 "name": "vrf02"
#             }
#         ]
#     },
#

# Using rendered:
- name: Render provided configuration
  arista.eos.eos_logging_global:
    config:
      format:
        timestamp:
          traditional:
            timezone: true
      level:
        facility: "AAA"
        severity: "alerts"
      persistent:
        size: 4096
      policy:
        invert_result: true
        match_list: "list01"
      hosts:
        - name: "host01"
          protocol: "tcp"
        - name: "11.11.11.1"
          port: 25
      vrfs:
        - name: "vrf01"
          source_interface: "Ethernet1"
        - name: "vrf02"
          hosts:
            - name: "hostvrf1"
              protocol: "tcp"
            - name: "24.1.1.1"
              port: "33"
# Module Execution:

# "rendered": [
#         "logging host host01 protocol tcp",
#         "logging host 11.11.11.1 25",
#         "logging vrf vrf01 source-interface Ethernet1",
#         "logging vrf vrf02 host hostvrf1 protocol tcp",
#         "logging vrf vrf02 host 24.1.1.1 33",
#         "logging format timestamp traditional timezone",
#         "logging level AAA alerts",
#         "logging persistent 4096",
#         "logging policy match invert-result match-list list01 discard"
#     ]
#
"""

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.arista.eos.plugins.module_utils.network.eos.argspec.logging_global.logging_global import (
    Logging_globalArgs,
)
from ansible_collections.arista.eos.plugins.module_utils.network.eos.config.logging_global.logging_global import (
    Logging_global,
)


def main():
    """
    Main entry point for module execution

    :returns: the result form module invocation
    """
    module = AnsibleModule(
        argument_spec=Logging_globalArgs.argument_spec,
        mutually_exclusive=[["config", "running_config"]],
        required_if=[
            ["state", "merged", ["config"]],
            ["state", "replaced", ["config"]],
            ["state", "overridden", ["config"]],
            ["state", "rendered", ["config"]],
            ["state", "parsed", ["running_config"]],
        ],
        supports_check_mode=True,
    )

    result = Logging_global(module).execute_module()
    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "certified"}

DOCUMENTATION = r"""
---
module: aci_syslog_remote_dest
short_description: Manage Syslog Remote Destinations (syslog:RemoteDest)
description:
- Manage remote destinations for syslog messages within an existing syslog group object.
options:
  admin_state:
    description:
    - Administrative state of the syslog remote destination
    type: str
    choices: [ enabled, disabled ]
  description:
    description:
    - Description of the remote destination
    type: str
  destination:
    description:
    - Hostname or IP address to send syslog messages to
    type: str
  format:
    description:
    - Format of the syslog messages
    type: str
    choices: [ aci, nxos ]
  facility:
    description:
    - Forwarding facility for syslog messages
    type: str
    choices: [ local0, local1, local2, local3, local4, local5, local6, local7 ]
  group:
    description:
    - Name of an existing syslog group
    type: str
    aliases: [ syslog_group, syslog_group_name ]
  mgmt_epg:
    description:
    - Name of a management EPG to send syslog messages from
    type: str
  name:
    description:
    - Name of the syslog remote destination
    type: str
    aliases: [ remote_destination_name, remote_destination ]
  severity:
    description:
    - Severity of messages to send to remote syslog
    type: str
    choices: [ alerts, critical, debugging, emergencies, error, information, notifications, warnings]
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    type: str
    choices: [ absent, present, query ]
    default: present
  syslog_port:
    description:
    - UDP port to send syslog messages to
    type: int
extends_documentation_fragment:
- cisco.aci.aci
- cisco.aci.annotation

seealso:
- name: APIC Management Information Model reference
  description: More information about the internal APIC classes B(syslog:RemoteDest).
  link: https://developer.cisco.com/docs/apic-mim-ref/
author:
- Tim Cragg (@timcragg)
"""

EXAMPLES = r"""
- name: Create a syslog remote destination
  cisco.aci.aci_syslog_remote_dest:
    host: apic
    username: admin
    password: SomeSecretPassword
    group: my_syslog_group
    facility: local7
    destination: 10.20.30.40
    syslog_port: 5678
    mgmt_epg: oob-default
    state: present
  delegate_to: localhost

- name: Delete syslog remote destination
  cisco.aci.aci_syslog_remote_dest:
    host: apic
    username: admin
    password: SomeSecretPassword
    group: my_syslog_group
    destination: 10.20.30.40
    state: absent
  delegate_to: localhost

- name: Query a syslog remote destination
  cisco.aci.aci_syslog_remote_dest:
    host: apic
    username: admin
    password: SomeSecretPassword
    group: my_syslog_group
    destination: 10.20.30.40
    state: query
  delegate_to: localhost
  register: query_result

- name: Query all syslog remote destinations
  cisco.aci.aci_syslog_remote_dest:
    host: apic
    username: admin
    password: SomeSecretPassword
    state: query
  delegate_to: localhost
  register: query_result
"""

RETURN = r"""
current:
  description: The existing configuration from the APIC after the module has finished
  returned: success
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production environment",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
error:
  description: The error information as returned from the APIC
  returned: failure
  type: dict
  sample:
    {
        "code": "122",
        "text": "unknown managed object class foo"
    }
raw:
  description: The raw output returned by the APIC REST API (xml or json)
  returned: parse error
  type: str
  sample: '<?xml version="1.0" encoding="UTF-8"?><imdata totalCount="1"><error code="122" text="unknown managed object class foo"/></imdata>'
sent:
  description: The actual/minimal configuration pushed to the APIC
  returned: info
  type: list
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment"
            }
        }
    }
previous:
  description: The original configuration from the APIC before the module has started
  returned: info
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
proposed:
  description: The assembled configuration from the user-provided parameters
  returned: info
  type: dict
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment",
                "name": "production"
            }
        }
    }
filter_string:
  description: The filter string used for the request
  returned: failure or debug
  type: str
  sample: ?rsp-prop-include=config-only
method:
  description: The HTTP method used for the request to the APIC
  returned: failure or debug
  type: str
  sample: POST
response:
  description: The HTTP response from the APIC
  returned: failure or debug
  type: str
  sample: OK (30 bytes)
status:
  description: The HTTP status from the APIC
  returned: failure or debug
  type: int
  sample: 200
url:
  description: The HTTP url used for the request to the APIC
  returned: failure or debug
  type: str
  sample: https://10.11.12.13/api/mo/uni/tn-production.json
"""

from ansible_collections.cisco.aci.plugins.module_utils.aci import ACIModule, aci_argument_spec, aci_annotation_spec
from ansible.module_utils.basic import AnsibleModule


def main():
    argument_spec = aci_argument_spec()
    argument_spec.update(aci_annotation_spec())
    argument_spec.update(
        name=dict(type="str", aliases=["remote_destination_name", "remote_destination"]),
        format=dict(type="str", choices=["aci", "nxos"]),
        admin_state=dict(type="str", choices=["enabled", "disabled"]),
        description=dict(type="str"),
        destination=dict(type="str"),
        facility=dict(type="str", choices=["local0", "local1", "local2", "local3", "local4", "local5", "local6", "local7"]),
        group=dict(type="str", aliases=["syslog_group", "syslog_group_name"]),
        mgmt_epg=dict(type="str"),
        syslog_port=dict(type="int"),
        severity=dict(type="str", choices=["alerts", "critical", "debugging", "emergencies", "error", "information", "notifications", "warnings"]),
        state=dict(type="str", default="present", choices=["absent", "present", "query"]),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["group", "destination"]],
            ["state", "present", ["group", "destination"]],
        ],
    )

    aci = ACIModule(module)

    name = module.params.get("name")
    format = module.params.get("format")
    admin_state = module.params.get("admin_state")
    description = module.params.get("description")
    destination = module.params.get("destination")
    facility = module.params.get("facility")
    group = module.params.get("group")
    syslog_port = module.params.get("syslog_port")
    severity = module.params.get("severity")
    state = module.params.get("state")
    mgmt_epg = module.params.get("mgmt_epg")

    aci.construct_url(
        root_class=dict(
            aci_class="syslogGroup",
            aci_rn="fabric/slgroup-{0}".format(group),
            module_object=group,
            target_filter={"name": group},
        ),
        subclass_1=dict(
            aci_class="syslogRemoteDest",
            aci_rn="rdst-{0}".format(destination),
            module_object=destination,
            target_filter={"host": destination},
        ),
        child_classes=["fileRsARemoteHostToEpg"],
    )

    aci.get_existing()

    if state == "present":
        child_configs = []
        if mgmt_epg:
            child_configs.append(
                dict(
                    fileRsARemoteHostToEpg=dict(
                        attributes=dict(tDn=("uni/tn-mgmt/mgmtp-default/{0}".format(mgmt_epg))),
                    ),
                )
            )
        aci.payload(
            aci_class="syslogRemoteDest",
            class_config=dict(
                adminState=admin_state,
                descr=description,
                format=format,
                forwardingFacility=facility,
                host=destination,
                name=name,
                port=syslog_port,
                severity=severity,
            ),
            child_configs=child_configs,
        )

        aci.get_diff(aci_class="syslogRemoteDest")

        aci.post_config()

    elif state == "absent":
        aci.delete_config()

    aci.exit_json()


if __name__ == "__main__":
    main()



# This code is part of Ansible, but is an independent component.
# This particular file snippet, and this file snippet only, is BSD licensed.
# Modules you write using this snippet, which is embedded dynamically by Ansible
# still belong to the author of the module, and may assign their own license
# to the complete work.
#
# Copyright (c) 2015 Peter Sprygada, <psprygada@ansible.com>
#
# Simplified BSD License (see LICENSES/BSD-2-Clause.txt or https://opensource.org/licenses/BSD-2-Clause)
# SPDX-License-Identifier: BSD-2-Clause

from __future__ import absolute_import, division, print_function


__metaclass__ = type

import re
import shlex
import time

from ansible.module_utils.parsing.convert_bool import BOOLEANS_FALSE, BOOLEANS_TRUE
from ansible.module_utils.six import string_types, text_type
from ansible.module_utils.six.moves import zip

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.utils import to_list


class FailedConditionsError(Exception):
    def __init__(self, msg, failed_conditions):
        super(FailedConditionsError, self).__init__(msg)
        self.failed_conditions = failed_conditions


class FailedConditionalError(Exception):
    def __init__(self, msg, failed_conditional):
        super(FailedConditionalError, self).__init__(msg)
        self.failed_conditional = failed_conditional


class AddCommandError(Exception):
    def __init__(self, msg, command):
        super(AddCommandError, self).__init__(msg)
        self.command = command


class AddConditionError(Exception):
    def __init__(self, msg, condition):
        super(AddConditionError, self).__init__(msg)
        self.condition = condition


class Cli(object):
    def __init__(self, connection):
        self.connection = connection
        self.default_output = connection.default_output or "text"
        self._commands = list()

    @property
    def commands(self):
        return [str(c) for c in self._commands]

    def __call__(self, commands, output=None):
        objects = list()
        for cmd in to_list(commands):
            objects.append(self.to_command(cmd, output))
        return self.connection.run_commands(objects)

    def to_command(self, command, output=None, prompt=None, response=None, **kwargs):
        output = output or self.default_output
        if isinstance(command, Command):
            return command
        if isinstance(prompt, string_types):
            prompt = re.compile(re.escape(prompt))
        return Command(command, output, prompt=prompt, response=response, **kwargs)

    def add_commands(self, commands, output=None, **kwargs):
        for cmd in commands:
            self._commands.append(self.to_command(cmd, output, **kwargs))

    def run_commands(self):
        responses = self.connection.run_commands(self._commands)
        for resp, cmd in zip(responses, self._commands):
            cmd.response = resp

        # wipe out the commands list to avoid issues if additional
        # commands are executed later
        self._commands = list()

        return responses


class Command(object):
    def __init__(self, command, output=None, prompt=None, response=None, **kwargs):
        self.command = command
        self.output = output
        self.command_string = command

        self.prompt = prompt
        self.response = response

        self.args = kwargs

    def __str__(self):
        return self.command_string


class CommandRunner(object):
    def __init__(self, module):
        self.module = module

        self.items = list()
        self.conditionals = set()

        self.commands = list()

        self.retries = 10
        self.interval = 1

        self.match = "all"

        self._default_output = module.connection.default_output

    def add_command(self, command, output=None, prompt=None, response=None, **kwargs):
        if command in [str(c) for c in self.commands]:
            raise AddCommandError("duplicated command detected", command=command)
        cmd = self.module.cli.to_command(
            command, output=output, prompt=prompt, response=response, **kwargs
        )
        self.commands.append(cmd)

    def get_command(self, command, output=None):
        for cmd in self.commands:
            if cmd.command == command:
                return cmd.response
        raise ValueError("command '%s' not found" % command)

    def get_responses(self):
        return [cmd.response for cmd in self.commands]

    def add_conditional(self, condition):
        try:
            self.conditionals.add(Conditional(condition))
        except AttributeError as exc:
            raise AddConditionError(msg=str(exc), condition=condition)

    def run(self):
        while self.retries > 0:
            self.module.cli.add_commands(self.commands)
            responses = self.module.cli.run_commands()

            for item in list(self.conditionals):
                if item(responses):
                    if self.match == "any":
                        return item
                    self.conditionals.remove(item)

            if not self.conditionals:
                break

            time.sleep(self.interval)
            self.retries -= 1
        else:
            failed_conditions = [item.raw for item in self.conditionals]
            errmsg = "One or more conditional statements have not been satisfied"
            raise FailedConditionsError(errmsg, failed_conditions)


class Conditional(object):
    """Used in command modules to evaluate waitfor conditions"""

    OPERATORS = {
        "eq": ["eq", "=="],
        "neq": ["neq", "ne", "!="],
        "gt": ["gt", ">"],
        "ge": ["ge", ">="],
        "lt": ["lt", "<"],
        "le": ["le", "<="],
        "contains": ["contains"],
        "matches": ["matches"],
    }

    def __init__(self, conditional, encoding=None):
        self.raw = conditional
        self.negate = False
        try:
            components = shlex.split(conditional)
            key, val = components[0], components[-1]
            op_components = components[1:-1]
            if "not" in op_components:
                self.negate = True
                op_components.pop(op_components.index("not"))
            op = op_components[0]

        except ValueError:
            raise ValueError("failed to parse conditional")

        self.key = key
        self.func = self._func(op)
        self.value = self._cast_value(val)

    def __call__(self, data):
        value = self.get_value(dict(result=data))
        if not self.negate:
            return self.func(value)
        else:
            return not self.func(value)

    def _cast_value(self, value):
        if value in BOOLEANS_TRUE:
            return True
        elif value in BOOLEANS_FALSE:
            return False
        elif re.match(r"^\d+\.d+$", value):
            return float(value)
        elif re.match(r"^\d+$", value):
            return int(value)
        else:
            return text_type(value)

    def _func(self, oper):
        for func, operators in self.OPERATORS.items():
            if oper in operators:
                return getattr(self, func)
        raise AttributeError("unknown operator: %s" % oper)

    def get_value(self, result):
        try:
            return self.get_json(result)
        except (IndexError, TypeError, AttributeError):
            msg = "unable to apply conditional to result"
            raise FailedConditionalError(msg, self.raw)

    def get_json(self, result):
        string = re.sub(r"\[[\'|\"]", ".", self.key)
        string = re.sub(r"[\'|\"]\]", ".", string)
        parts = re.split(r"\.(?=[^\]]*(?:\[|$))", string)
        for part in parts:
            match = re.findall(r"\[(\S+?)\]", part)
            if match:
                key = part[: part.find("[")]
                result = result[key]
                for m in match:
                    try:
                        m = int(m)
                    except ValueError:
                        m = str(m)
                    result = result[m]
            else:
                result = result.get(part)
        return result

    def number(self, value):
        if "." in str(value):
            return float(value)
        else:
            return int(value)

    def eq(self, value):
        return value == self.value

    def neq(self, value):
        return value != self.value

    def gt(self, value):
        return self.number(value) > self.value

    def ge(self, value):
        return self.number(value) >= self.value

    def lt(self, value):
        return self.number(value) < self.value

    def le(self, value):
        return self.number(value) <= self.value

    def contains(self, value):
        return str(self.value) in value

    def matches(self, value):
        match = re.search(self.value, value, re.M)
        return match is not None



#!/usr/bin/python
# coding: utf-8 -*-

# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1', 'status': ['preview'], 'supported_by': 'community'}

DOCUMENTATION = '''
---
module: workflow_launch
author: "John Westcott IV (@john-westcott-iv)"
short_description: Run a workflow in Automation Platform Controller
description:
    - Launch an Automation Platform Controller workflows. See
      U(https://www.ansible.com/tower) for an overview.
options:
    name:
      description:
        - The name of the workflow template to run.
      required: True
      type: str
      aliases:
        - workflow_template
    organization:
      description:
        - Organization name, ID, or named URL the workflow job template exists in.
        - Used to help lookup the object, cannot be modified using this module.
        - If not provided, will lookup by name only, which does not work with duplicates.
      type: str
    inventory:
      description:
        - Inventory name, ID, or named URL to use for the job ran with this workflow, only used if prompt for inventory is set.
      type: str
    limit:
      description:
        - Limit to use for the I(job_template).
      type: str
    tags:
      description:
        - Specific tags to apply from the I(job_template).
      type: list
      elements: str
    skip_tags:
      description:
        - Specific tags to skip from the I(job_template).
      type: list
      elements: str
    scm_branch:
      description:
        - A specific branch of the SCM project to run the template on.
        - This is only applicable if your project allows for branch override.
      type: str
    extra_vars:
      description:
        - Any extra vars required to launch the job.
      type: dict
    wait:
      description:
        - Wait for the workflow to complete.
      default: True
      type: bool
    interval:
      description:
        - The interval to request an update from the controller.
      required: False
      default: 2
      type: float
    timeout:
      description:
        - If waiting for the workflow to complete this will abort after this
          amount of seconds
      type: int
extends_documentation_fragment: awx.awx.auth
'''

RETURN = '''
job_info:
    description: dictionary containing information about the workflow executed
    returned: If workflow launched
    type: dict
'''


EXAMPLES = '''
- name: Launch a workflow with a timeout of 10 seconds
  workflow_launch:
    workflow_template: "Test Workflow"
    timeout: 10

- name: Launch a Workflow with extra_vars without waiting
  workflow_launch:
    workflow_template: "Test workflow"
    extra_vars:
      var1: My First Variable
      var2: My Second Variable
    wait: False
'''

from ..module_utils.controller_api import ControllerAPIModule


def main():
    # Any additional arguments that are not fields of the item can be added here
    argument_spec = dict(
        name=dict(required=True, aliases=['workflow_template']),
        organization=dict(),
        inventory=dict(),
        limit=dict(),
        tags=dict(type='list', elements='str'),
        skip_tags=dict(type='list', elements='str'),
        scm_branch=dict(),
        extra_vars=dict(type='dict'),
        wait=dict(required=False, default=True, type='bool'),
        interval=dict(required=False, default=2.0, type='float'),
        timeout=dict(required=False, type='int'),
    )

    # Create a module for ourselves
    module = ControllerAPIModule(argument_spec=argument_spec)

    optional_args = {}
    # Extract our parameters
    name = module.params.get('name')
    organization = module.params.get('organization')
    inventory = module.params.get('inventory')
    wait = module.params.get('wait')
    interval = module.params.get('interval')
    timeout = module.params.get('timeout')

    for field_name in (
        'limit',
        'extra_vars',
        'scm_branch',
    ):
        field_val = module.params.get(field_name)
        if field_val is not None:
            optional_args[field_name] = field_val

    # Special treatment of tags parameters
    job_tags = module.params.get('tags')
    if job_tags is not None:
        optional_args['job_tags'] = ",".join(job_tags)
    skip_tags = module.params.get('skip_tags')
    if skip_tags is not None:
        optional_args['skip_tags'] = ",".join(skip_tags)

    # Create a datastructure to pass into our job launch
    post_data = {}
    for arg_name, arg_value in optional_args.items():
        if arg_value:
            post_data[arg_name] = arg_value

    # Attempt to look up the related items the user specified (these will fail the module if not found)
    if inventory:
        post_data['inventory'] = module.resolve_name_to_id('inventories', inventory)

    # Attempt to look up job_template based on the provided name
    lookup_data = {}
    if organization:
        lookup_data['organization'] = module.resolve_name_to_id('organizations', organization)
    workflow_job_template = module.get_one('workflow_job_templates', name_or_id=name, data=lookup_data)

    if workflow_job_template is None:
        module.fail_json(msg="Unable to find workflow job template")

    # The API will allow you to submit values to a jb launch that are not prompt on launch.
    # Therefore, we will test to see if anything is set which is not prompt on launch and fail.
    check_vars_to_prompts = {
        'inventory': 'ask_inventory_on_launch',
        'limit': 'ask_limit_on_launch',
        'job_tags': 'ask_tags_on_launch',
        'skip_tags': 'ask_skip_tags_on_launch',
        'scm_branch': 'ask_scm_branch_on_launch',
    }

    param_errors = []
    for variable_name, prompt in check_vars_to_prompts.items():
        if variable_name in post_data and not workflow_job_template[prompt]:
            param_errors.append("The field {0} was specified but the workflow job template does not allow for it to be overridden".format(variable_name))
    # Check if Either ask_variables_on_launch, or survey_enabled is enabled for use of extra vars.
    if module.params.get('extra_vars') and not (workflow_job_template['ask_variables_on_launch'] or workflow_job_template['survey_enabled']):
        param_errors.append("The field extra_vars was specified but the workflow job template does not allow for it to be overridden")
    if len(param_errors) > 0:
        module.fail_json(msg="Parameters specified which can not be passed into workflow job template, see errors for details", errors=param_errors)

    # Launch the job
    result = module.post_endpoint(workflow_job_template['related']['launch'], data=post_data)

    if result['status_code'] != 201:
        module.fail_json(msg="Failed to launch workflow, see response for details", response=result)

    module.json_output['changed'] = True
    module.json_output['id'] = result['json']['id']
    module.json_output['status'] = result['json']['status']
    # This is for backwards compatability
    module.json_output['job_info'] = {'id': result['json']['id']}

    if not wait:
        module.exit_json(**module.json_output)

    # Invoke wait function
    module.wait_on_url(url=result['json']['url'], object_name=name, object_type='Workflow Job', timeout=timeout, interval=interval)

    module.exit_json(**module.json_output)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: applications_v2_info
short_description: Information module for Applications V2
description:
- Get all Applications V2.
- Get application/s by offset/limit or by name.
version_added: '6.14.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  attributes:
    description:
    - Attributes query parameter. Attributes to retrieve, valid value application.
    type: str
  name:
    description:
    - Name query parameter. The application name.
    type: str
  offset:
    description:
    - Offset query parameter. The starting point or index from where the paginated results should begin.
    type: float
  limit:
    description:
    - >
      Limit query parameter. The limit which is the maximum number of items to include in a single page of
      results, max value 500.
    type: float
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Application Policy GetApplicationsV2
  description: Complete reference of the GetApplicationsV2 API.
  link: https://developer.cisco.com/docs/dna-center/#!get-applications-v-2
notes:
  - SDK Method used are
    application_policy.ApplicationPolicy.get_applications_v2,

  - Paths used are
    get /dna/intent/api/v2/applications,

"""

EXAMPLES = r"""
- name: Get all Applications V2
  cisco.dnac.applications_v2_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    attributes: string
    name: string
    offset: 0
    limit: 0
  register: result

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": [
        {
          "id": "string",
          "instanceId": 0,
          "displayName": "string",
          "instanceVersion": 0,
          "identitySource": {
            "id": "string",
            "type": "string"
          },
          "indicativeNetworkIdentity": [
            {
              "id": "string",
              "displayName": "string",
              "lowerPort": 0,
              "ports": "string",
              "protocol": "string",
              "upperPort": 0
            }
          ],
          "name": "string",
          "namespace": "string",
          "networkApplications": [
            {
              "id": "string",
              "appProtocol": "string",
              "applicationSubType": "string",
              "applicationType": "string",
              "categoryId": "string",
              "displayName": "string",
              "dscp": "string",
              "engineId": "string",
              "helpString": "string",
              "longDescription": "string",
              "name": "string",
              "popularity": 0,
              "rank": 0,
              "selectorId": "string",
              "serverName": "string",
              "url": "string",
              "trafficClass": "string"
            }
          ],
          "networkIdentity": [
            {
              "id": "string",
              "displayName": "string",
              "ipv4Subnet": [
                "string"
              ],
              "ipv6Subnet": [
                {}
              ],
              "lowerPort": 0,
              "ports": "string",
              "protocol": "string",
              "upperPort": 0
            }
          ],
          "parentScalableGroup": {
            "id": "string",
            "idRef": "string"
          },
          "qualifier": "string",
          "scalableGroupExternalHandle": "string",
          "scalableGroupType": "string",
          "type": "string"
        }
      ],
      "version": "string"
    }
"""



# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

"""
filter plugin file for ipaddr filters: ip4_hex
"""
from __future__ import absolute_import, division, print_function

from functools import partial

from ansible.errors import AnsibleFilterError

from ansible_collections.ansible.utils.plugins.module_utils.common.argspec_validate import (
    AnsibleArgSpecValidator,
)
from ansible_collections.ansible.utils.plugins.plugin_utils.base.ipaddr_utils import _need_netaddr


__metaclass__ = type


try:
    from jinja2.filters import pass_environment
except ImportError:
    from jinja2.filters import environmentfilter as pass_environment

try:
    import netaddr

    HAS_NETADDR = True
except ImportError:
    # in this case, we'll make the filters return error messages (see bottom)
    HAS_NETADDR = False
else:

    class mac_linux(netaddr.mac_unix):
        pass

    mac_linux.word_fmt = "%.2x"

DOCUMENTATION = """
    name: ip4_hex
    author: Ashwini Mhatre (@amhatre)
    version_added: "2.5.0"
    short_description: This filter is designed to convert IPv4 address to Hexadecimal notation with optional delimiter.
    description:
        - This filter convert IPv4 address to Hexadecimal notation with optional delimiter
    options:
        arg:
            description: IPv4 address.
            type: str
            required: True
        delimiter:
            description:
            - You can provide a single argument to each ip4_hex() filter as delimiter.
            type: str
            default: ''
    notes:
"""

EXAMPLES = r"""
#### examples
# ip4_hex convert IPv4 address to Hexadecimal notation with optional delimiter
- debug:
    msg: "{{ '192.168.1.5' | ansible.utils.ip4_hex }}"

# ip4_hex with delimiter
- debug:
    msg: "{{ '192.168.1.5' | ansible.utils.ip4_hex(':') }}"

# TASK [debug] ************************************************************************************************
# task path: /Users/amhatre/ansible-collections/playbooks/test_ip4_hex.yaml:7
# Loading collection ansible.utils from /Users/amhatre/ansible-collections/collections/ansible_collections/ansible/utils
# ok: [localhost] => {
#     "msg": "c0a80105"
# }
#
# TASK [debug] ************************************************************************************************
# task path: /Users/amhatre/ansible-collections/playbooks/test_ip4_hex.yaml:11
# Loading collection ansible.utils from /Users/amhatre/ansible-collections/collections/ansible_collections/ansible/utils
# ok: [localhost] => {
#     "msg": "c0:a8:01:05"
# }
"""

RETURN = """
  data:
    type: str
    description:
      - Returns IPv4 address to Hexadecimal notation.
"""


@pass_environment
def _ip4_hex(*args, **kwargs):
    """This filter is designed to Convert an IPv4 address to Hexadecimal notation"""
    keys = ["arg", "delimiter"]
    data = dict(zip(keys, args[1:]))
    data.update(kwargs)
    aav = AnsibleArgSpecValidator(data=data, schema=DOCUMENTATION, name="ip4_hex")
    valid, errors, updated_data = aav.validate()
    if not valid:
        raise AnsibleFilterError(errors)
    return ip4_hex(**updated_data)


def ip4_hex(arg, delimiter=""):
    """Convert an IPv4 address to Hexadecimal notation"""
    try:
        ip = netaddr.IPAddress(arg)
    except (netaddr.AddrFormatError, ValueError):
        msg = "You must pass a valid IP address; {0} is invalid".format(arg)
        raise AnsibleFilterError(msg)
    numbers = list(map(int, arg.split(".")))
    return "{0:02x}{sep}{1:02x}{sep}{2:02x}{sep}{3:02x}".format(*numbers, sep=delimiter)


class FilterModule(object):
    """IP address and network manipulation filters"""

    filter_map = {
        # IP addresses and networks
        "ip4_hex": _ip4_hex,
    }

    def filters(self):
        """ip4_hex filter"""
        if HAS_NETADDR:
            return self.filter_map
        else:
            return dict((f, partial(_need_netaddr, f)) for f in self.filter_map)



#!/usr/bin/python
# coding: utf-8 -*-

# (c) 2017, Wayne Witzel III <wayne@riotousliving.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type


ANSIBLE_METADATA = {'metadata_version': '1.1', 'status': ['preview'], 'supported_by': 'community'}


DOCUMENTATION = '''
---
module: ad_hoc_command_cancel
author: "John Westcott IV (@john-westcott-iv)"
short_description: Cancel an Ad Hoc Command.
description:
    - Cancel ad hoc command. See
      U(https://www.ansible.com/tower) for an overview.
options:
    command_id:
      description:
        - ID of the command to cancel
      required: True
      type: int
    fail_if_not_running:
      description:
        - Fail loudly if the I(command_id) can not be canceled
      default: False
      type: bool
    interval:
      description:
        - The interval in seconds, to request an update from .
      required: False
      default: 1
      type: float
    timeout:
      description:
        - Maximum time in seconds to wait for a job to finish.
        - Not specifying means the task will wait until the controller cancels the command.
      type: int
      default: 0
extends_documentation_fragment: awx.awx.auth
'''

EXAMPLES = '''
- name: Cancel command
  ad_hoc_command_cancel:
    command_id: command.id
'''

RETURN = '''
id:
    description: command id requesting to cancel
    returned: success
    type: int
    sample: 94
'''


import time

from ..module_utils.controller_api import ControllerAPIModule


def main():
    # Any additional arguments that are not fields of the item can be added here
    argument_spec = dict(
        command_id=dict(type='int', required=True),
        fail_if_not_running=dict(type='bool', default=False),
        interval=dict(type='float', default=1.0),
        timeout=dict(type='int', default=0),
    )

    # Create a module for ourselves
    module = ControllerAPIModule(argument_spec=argument_spec)

    # Extract our parameters
    command_id = module.params.get('command_id')
    fail_if_not_running = module.params.get('fail_if_not_running')
    interval = module.params.get('interval')
    timeout = module.params.get('timeout')

    # Attempt to look up the command based on the provided name
    command = module.get_one(
        'ad_hoc_commands',
        **{
            'data': {
                'id': command_id,
            }
        }
    )

    if command is None:
        module.fail_json(msg="Unable to find command with id {0}".format(command_id))

    cancel_page = module.get_endpoint(command['related']['cancel'])
    if 'json' not in cancel_page or 'can_cancel' not in cancel_page['json']:
        module.fail_json(msg="Failed to cancel command, got unexpected response", **{'response': cancel_page})

    if not cancel_page['json']['can_cancel']:
        if fail_if_not_running:
            module.fail_json(msg="Ad Hoc Command is not running")
        else:
            module.exit_json(**{'changed': False})

    results = module.post_endpoint(command['related']['cancel'], **{'data': {}})

    if results['status_code'] != 202:
        module.fail_json(msg="Failed to cancel command, see response for details", **{'response': results})

    result = module.get_endpoint(command['related']['cancel'])
    start = time.time()
    while result['json']['can_cancel']:
        # If we are past our time out fail with a message
        if timeout and timeout < time.time() - start:
            # Account for Legacy messages
            module.json_output['msg'] = 'Monitoring of ad hoc command aborted due to timeout'
            module.fail_json(**module.json_output)

        # Put the process to sleep for our interval
        time.sleep(interval)

        result = module.get_endpoint(command['related']['cancel'])

    module.exit_json(**{'changed': True})


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)


DOCUMENTATION = r"""
---
module: event_subscription_details_syslog_info
short_description: Information module for Event Subscription Details Syslog Info
description:
- This module represents an alias of the module event_subscription_details_syslog_v1_info
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  name:
    description:
    - Name query parameter. Name of the specific configuration.
    type: str
  instanceId:
    description:
    - InstanceId query parameter. Instance Id of the specific configuration.
    type: str
  offset:
    description:
    - >
      Offset query parameter. The number of Syslog Subscription detail's to offset in the resultset whose default
      value 0.
    type: float
  limit:
    description:
    - Limit query parameter. The number of Syslog Subscription detail's to limit in the resultset whose default value 10.
    type: float
  sortBy:
    description:
    - SortBy query parameter. SortBy field name.
    type: str
  order:
    description:
    - Order query parameter.
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Event Management GetSyslogSubscriptionDetailsV1
  description: Complete reference of the GetSyslogSubscriptionDetailsV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!get-syslog-subscription-details
notes:
  - SDK Method used are
    event_management.EventManagement.get_syslog_subscription_details_v1,

  - Paths used are
    get /dna/intent/api/v1/event/subscription-details/syslog,
  - It should be noted that this module is an alias of event_subscription_details_syslog_v1_info

"""

EXAMPLES = r"""
- name: Get all Event Subscription Details Syslog Info
  cisco.dnac.event_subscription_details_syslog_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    name: string
    instanceId: string
    offset: 0
    limit: 0
    sortBy: string
    order: string
  register: result

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: list
  elements: dict
  sample: >
    [
      {
        "instanceId": "string",
        "name": "string",
        "description": "string",
        "connectorType": "string",
        "syslogConfig": {
          "configId": "string",
          "name": "string",
          "description": "string",
          "host": "string",
          "port": "string",
          "protocol": "string"
        }
      }
    ]
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: reservation
short_description: Resource module for Reservation
description:
- Manage operations create, update and delete of the resource Reservation.
- Reserve given number of SGTs in a continuous range for the given Client.
- Delete the reserved range of SGT for the given Client.
- Update the reserved ranges of a specific Client by giving the custom start and end index.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.ise.module
author: Rafael Campos (@racampos)
options:
  clientID:
    description: ClientID path parameter. Unique name for a Client.
    type: str
  clientName:
    description: Name of the given client.
    type: str
  endIndex:
    description: End index of the reserved range.
    type: int
  numberOfTags:
    description: Nummber of tags required to be reserved in ISE.
    type: int
  startIndex:
    description: Start index of the reserved range.
    type: int
requirements:
- ciscoisesdk >= 2.2.3
- python >= 3.5
notes:
  - SDK Method used are
    sgt_range_reservation.SgtRangeReservation.delete_sgt_reserve_range,
    sgt_range_reservation.SgtRangeReservation.reserve_sgt_range,
    sgt_range_reservation.SgtRangeReservation.update_reserved_range,

  - Paths used are
    post /api/v1/sgt/reservation/reserveRange,
    delete /api/v1/sgt/reservation/{clientID},
    put /api/v1/sgt/reservation/{clientID},

"""

EXAMPLES = r"""
- name: Create
  cisco.ise.reservation:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    state: present
    clientName: string
    numberOfTags: 0

- name: Update by id
  cisco.ise.reservation:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    state: present
    clientID: string
    endIndex: 0
    startIndex: 0

- name: Delete by id
  cisco.ise.reservation:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    state: absent
    clientID: string

"""

RETURN = r"""
ise_response:
  description: A dictionary or list with the response returned by the Cisco ISE Python SDK
  returned: always
  type: dict
  sample: >
    {
      "clientID": "string",
      "clientName": "string",
      "endIndex": 0,
      "startIndex": 0
    }

ise_update_response:
  description: A dictionary or list with the response returned by the Cisco ISE Python SDK
  returned: always
  version_added: '1.1.0'
  type: dict
  sample: >
    {
      "success": {
        "message": "string"
      },
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: trustsec_vn_bulk_update
short_description: Resource module for Trustsec VN Bulk Update
description:
- Manage operation create of the resource Trustsec VN Bulk Update.
version_added: '2.0.0'
extends_documentation_fragment:
  - cisco.ise.module
author: Rafael Campos (@racampos)
options:
  payload:
    description: Trustsec VN Bulk Update's payload.
    elements: dict
    suboptions:
      additionalAttributes:
        description: JSON String of additional attributes for the Virtual Network.
        type: str
      id:
        description: Identifier of the Virtual Network.
        type: str
      lastUpdate:
        description: Timestamp for the last update of the Virtual Network.
        type: str
      name:
        description: Name of the Virtual Network.
        type: str
    type: list
requirements:
- ciscoisesdk >= 2.2.3
- python >= 3.5
seealso:
- name: Cisco ISE documentation for virtualNetwork
  description: Complete reference of the virtualNetwork API.
  link: https://developer.cisco.com/docs/identity-services-engine/v1/#!trustsec-openapi
notes:
  - SDK Method used are
    virtual_network.VirtualNetwork.bulk_update_virtual_networks,

  - Paths used are
    post /api/v1/trustsec/virtualnetwork/bulk/update,

"""

EXAMPLES = r"""
- name: Create
  cisco.ise.trustsec_vn_bulk_update:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    payload:
    - additionalAttributes: string
      id: string
      lastUpdate: string
      name: string

"""

RETURN = r"""
ise_response:
  description: A dictionary or list with the response returned by the Cisco ISE Python SDK
  returned: always
  type: dict
  sample: >
    {
      "id": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: sp_profile_delete_v2
short_description: Resource module for Sp Profile Delete V2
description:
- Manage operation delete of the resource Sp Profile Delete V2.
- API to delete Service Provider Profile QoS .
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  spProfileName:
    description: SpProfileName path parameter. SP profile name.
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Network Settings DeleteSPProfileV2
  description: Complete reference of the DeleteSPProfileV2 API.
  link: https://developer.cisco.com/docs/dna-center/#!delete-sp-profile-v-2
notes:
  - SDK Method used are
    network_settings.NetworkSettings.delete_sp_profile_v2,

  - Paths used are
    delete /dna/intent/api/v2/sp-profile/{spProfileName},

"""

EXAMPLES = r"""
- name: Delete by name
  cisco.dnac.sp_profile_delete_v2:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    spProfileName: string

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "taskId": "string",
        "url": "string"
      },
      "version": "string"
    }
"""



#!/usr/bin/python
#
# Copyright (c) 2023 xuzhang3 (@xuzhang3), Fred-sun (@Fred-sun)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = '''
---
module: azure_rm_batchaccount_info
version_added: "0.1.2"
short_description: Get the Batch Account on Azure facts
description:
    - Get the Batch Account on Azure facts.

options:
    resource_group:
        description:
            - The name of the resource group in which to create the Batch Account.
        type: str
    name:
        description:
            - The name of the Batch Account.
        type: str
    tags:
        description:
            - Limit results by providing a list of tags. Format tags as 'key' or 'key:value'.
        type: list
        elements: str

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - xuzhang3 (@xuzhang3)
    - Fred Sun (@Fred-sun)
'''

EXAMPLES = '''
- name: Get the Batch Account by name
  azure_rm_batchaccount_info:
    resource_group: MyResGroup
    name: mybatchaccount

- name: List the Batch Account by subscription
  azure_rm_batchaccount_info:
    tags:
      - key1
'''

RETURN = '''
id:
    description:
        - The ID of the Batch account.
    returned: always
    type: str
    sample: "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/myResourceGroup/providers/Microsoft.Batch/batchAccounts/sampleacct"
account_endpoint:
    description:
        - The account endpoint used to interact with the Batch service.
    returned: always
    type: str
    sample: sampleacct.westus.batch.azure.com
'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_ext import AzureRMModuleBaseExt

try:
    from azure.core.exceptions import ResourceNotFoundError
except ImportError:
    # This is handled in azure_rm_common
    pass


class AzureRMBatchAccountInfo(AzureRMModuleBaseExt):
    """Configuration class for an Azure RM Batch Account resource"""

    def __init__(self):
        self.module_arg_spec = dict(
            resource_group=dict(
                type='str',
            ),
            name=dict(
                type='str',
            ),
            tags=dict(
                type='list',
                elements='str'
            )
        )

        self.resource_group = None
        self.name = None
        self.tags = None

        self.results = dict(changed=False)

        super(AzureRMBatchAccountInfo, self).__init__(derived_arg_spec=self.module_arg_spec,
                                                      supports_check_mode=True,
                                                      supports_tags=False)

    def exec_module(self, **kwargs):
        """Main module execution method"""

        for key in list(self.module_arg_spec.keys()) + ['tags']:
            setattr(self, key, kwargs[key])

        response = []

        if self.resource_group is not None and self.name is not None:
            response = [self.get_batchaccount()]
        elif self.resource_group is not None:
            response = self.list_by_resourcegroup()
        else:
            response = self.list_all()

        self.results['batch_account'] = [self.format_item(item) for item in response if item and self.has_tags(item.get('tags'), self.tags)]

        return self.results

    def list_by_resourcegroup(self):
        self.log("List all Batch Account in the rsource group {0}".format(self.resource_group))
        result = []
        response = []
        try:
            response = self.batch_account_client.batch_account.list_by_resource_group(resource_group_name=self.resource_group)
            self.log("Response : {0}".format(response))
        except Exception as e:
            self.log('Did not find the Batch Account instance. Exception as {0}'.format(e))
        for item in response:
            result.append(item.as_dict())
        return result

    def list_all(self):
        self.log("List all Batch Account in the same subscritpion")
        result = []
        response = []
        try:
            response = self.batch_account_client.batch_account.list()
            self.log("Response : {0}".format(response))
        except Exception as e:
            self.log('Did not find the Batch Account instance.')
        for item in response:
            result.append(item.as_dict())
        return result

    def get_batchaccount(self):
        '''
        Gets the properties of the specified Batch Account
        :return: deserialized Batch Account instance state dictionary
        '''
        self.log("Checking if the Batch Account instance {0} is present".format(self.name))
        try:
            response = self.batch_account_client.batch_account.get(resource_group_name=self.resource_group,
                                                                   account_name=self.name)
            self.log("Response : {0}".format(response))
            self.log("Batch Account instance : {0} found".format(response.name))
        except ResourceNotFoundError as e:
            self.log('Did not find the Batch Account instance.')
            return
        return response.as_dict()

    def format_item(self, item):
        result = {
            'id': item['id'],
            'name': item['name'],
            'type': item['type'],
            'location': item['location'],
            'account_endpoint': item['account_endpoint'],
            'provisioning_state': item['provisioning_state'],
            'pool_allocation_mode': item['pool_allocation_mode'],
            'auto_storage': item['auto_storage'],
            'dedicated_core_quota': item['dedicated_core_quota'],
            'low_priority_core_quota': item['low_priority_core_quota'],
            'pool_quota': item['pool_quota'],
            'active_job_and_job_schedule_quota': item['active_job_and_job_schedule_quota'],
            'tags': item.get('tags'),
            'identity': item.get('identity')
        }
        return result


def main():
    """Main execution"""
    AzureRMBatchAccountInfo()


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
# Copyright 2019 Red Hat
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
"""
The arg spec for the eos facts module.
"""

from __future__ import absolute_import, division, print_function


__metaclass__ = type


class FactsArgs(object):
    """The arg spec for the eos facts module"""

    def __init__(self, **kwargs):
        pass

    argument_spec = {
        "gather_subset": dict(default=["min"], type="list", elements="str"),
        "gather_network_resources": dict(type="list", elements="str"),
        "available_network_resources": {"type": "bool", "default": False},
    }



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)


DOCUMENTATION = r"""
---
module: sites_bulk
short_description: Resource module for Sites Bulk
description:
- This module represents an alias of the module sites_bulk_v1
version_added: '6.15.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  payload:
    description: Sites Bulk's payload.
    elements: dict
    suboptions:
      address:
        description: Building address. Example 4900 Marie P. Debartolo Way, Santa Clara,
          California 95054, United States.
        type: str
      country:
        description: Country name. Required for building.
        type: str
      floorNumber:
        description: Floor number. Required for floor.
        type: int
      height:
        description: Floor height. Required for floor. Example 10.1.
        type: float
      latitude:
        description: Building Latitude. Example 37.403712.
        type: float
      length:
        description: Floor length. Required for floor. Example 110.3.
        type: float
      longitude:
        description: Building Longitude. Example -121.971063.
        type: float
      name:
        description: Site name.
        type: str
      parentNameHierarchy:
        description: Parent hierarchical name. Example Global/USA/San Jose/Building1.
        type: str
      rfModel:
        description: Floor RF Model. Required for floor.
        type: str
      type:
        description: Type.
        type: str
      unitsOfMeasure:
        description: Floor unit of measure. Required for floor.
        type: str
      width:
        description: Floor width. Required for floor. Example 100.5.
        type: float
    type: list
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Site Design CreateSitesV1
  description: Complete reference of the CreateSitesV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!create-sites
notes:
  - SDK Method used are
    site_design.SiteDesign.create_sites_v1,

  - Paths used are
    post /dna/intent/api/v1/sites/bulk,
  - It should be noted that this module is an alias of sites_bulk_v1

"""

EXAMPLES = r"""
- name: Create
  cisco.dnac.sites_bulk:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    payload:
    - address: string
      country: string
      floorNumber: 0
      height: 0
      latitude: 0
      length: 0
      longitude: 0
      name: string
      parentNameHierarchy: string
      rfModel: string
      type: string
      unitsOfMeasure: string
      width: 0

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "version": "string",
      "response": {
        "url": "string",
        "taskId": "string"
      }
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)


DOCUMENTATION = r"""
---
module: wireless_settings_dot11be_profiles
short_description: Resource module for Wireless Settings Dot11Be Profiles
description:
- This module represents an alias of the module wireless_settings_dot11be_profiles_v1
version_added: '6.15.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  id:
    description: Id path parameter. 802.11be Profile ID.
    type: str
  muMimoDownLink:
    description: MU-MIMO Downlink (Default false).
    type: bool
  muMimoUpLink:
    description: MU-MIMO Uplink (Default false).
    type: bool
  ofdmaDownLink:
    description: OFDMA Downlink (Default true).
    type: bool
  ofdmaMultiRu:
    description: OFDMA Multi-RU (Default false).
    type: bool
  ofdmaUpLink:
    description: OFDMA Uplink (Default true).
    type: bool
  profileName:
    description: 802.11be Profile Name.
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Wireless CreateA80211beProfileV1
  description: Complete reference of the CreateA80211beProfileV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!create-a-80-21-1be-profile
- name: Cisco DNA Center documentation for Wireless DeleteA80211beProfileV1
  description: Complete reference of the DeleteA80211beProfileV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!delete-a-80-21-1be-profile
- name: Cisco DNA Center documentation for Wireless Update80211beProfileV1
  description: Complete reference of the Update80211beProfileV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!update-80-21-1be-profile
notes:
  - SDK Method used are
    wireless.Wireless.create_a80211be_profile_v1,
    wireless.Wireless.delete_a80211be_profile_v1,
    wireless.Wireless.update80211be_profile_v1,

  - Paths used are
    post /dna/intent/api/v1/wirelessSettings/dot11beProfiles,
    delete /dna/intent/api/v1/wirelessSettings/dot11beProfiles/{id},
    put /dna/intent/api/v1/wirelessSettings/dot11beProfiles/{id},
  - It should be noted that this module is an alias of wireless_settings_dot11be_profiles_v1

"""

EXAMPLES = r"""
- name: Create
  cisco.dnac.wireless_settings_dot11be_profiles:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    muMimoDownLink: true
    muMimoUpLink: true
    ofdmaDownLink: true
    ofdmaMultiRu: true
    ofdmaUpLink: true
    profileName: string

- name: Delete by id
  cisco.dnac.wireless_settings_dot11be_profiles:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: absent
    id: string

- name: Update by id
  cisco.dnac.wireless_settings_dot11be_profiles:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    id: string
    muMimoDownLink: true
    muMimoUpLink: true
    ofdmaDownLink: true
    ofdmaMultiRu: true
    ofdmaUpLink: true
    profileName: string

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "taskId": "string",
        "url": "string"
      },
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: duo_identity_sync_info
short_description: Information module for Duo Identity Sync Info
description:
- Get all Duo Identity Sync Info.
- Get Duo Identity Sync Info by name.
- Duo-IdentitySync - Get the Identitysync config specified in the syncName.
- Duo-IdentitySync - Get the list of all Identitysync configurations.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.ise.module_info
author: Rafael Campos (@racampos)
options:
  syncName:
    description:
    - SyncName path parameter. This name is used to update, delete or retrieve the specific Identitysync config.
    type: str
requirements:
- ciscoisesdk >= 2.2.3
- python >= 3.5
notes:
  - SDK Method used are
    duo_identity_sync.DuoIdentitySync.get_identitysync,
    duo_identity_sync.DuoIdentitySync.get_identitysync_by_sync_name,

  - Paths used are
    get /api/v1/duo-identitysync/identitysync,
    get /api/v1/duo-identitysync/identitysync/{syncName},

"""

EXAMPLES = r"""
- name: Get all Duo Identity Sync Info
  cisco.ise.duo_identity_sync_info:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
  register: result

- name: Get Duo Identity Sync Info by name
  cisco.ise.duo_identity_sync_info:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    syncName: string
  register: result

"""

RETURN = r"""
ise_response:
  description: A dictionary or list with the response returned by the Cisco ISE Python SDK
  returned: always
  type: dict
  sample: >
    {
      "identitySync": {
        "adGroups": [
          {
            "name": "string",
            "sid": "string",
            "source": "string"
          }
        ],
        "configurations": {
          "activeDirectories": [
            {
              "directoryID": "string",
              "domain": "string",
              "name": "string"
            }
          ]
        },
        "lastSync": "string",
        "syncName": "string",
        "syncSchedule": {
          "interval": 0,
          "intervalUnit": "string",
          "schedulerSync": "string",
          "startDate": "string"
        },
        "syncStatus": "string"
      }
    }
"""
